import re
import pandas as pd
from retry import retry
from ..utils import requests_obj, validate_request, to_numeric
from ..common.getter import BaseApplication
from ..common.explain_change import exchange_explain


@validate_request
@to_numeric
@retry(tries=3)
def get_fund_codes(token: str, ft: str = None, is_explain: bool = False) -> pd.DataFrame:
    """
    获取天天基金网公开的全部公墓基金名单
    Parameters
    ----------
    ft : str, optional
        基金类型可选示例如下
        - ``'zq'``  : 债券类型基金
        - ``'gp'``  : 股票类型基金
        - ``'etf'`` : ETF 基金
        - ``'hh'``  : 混合型基金
        - ``'zs'``  : 指数型基金
        - ``'fof'`` : FOF 基金
        - ``'qdii'``: QDII 型基金
        - ``None``  : 全部
    Returns
    -------
    DataFrame
        天天基金网基金名单数据
    Examples
    --------
        基金代码                  基金简称
    0     003834              华夏能源革新股票
    1     005669            前海开源公用事业股票
    2     004040             金鹰医疗健康产业A
    3     517793                 1.20%
    4     004041             金鹰医疗健康产业C
    ...      ...                   ...
    1981  012503      国泰中证环保产业50ETF联接A
    1982  012517  国泰中证细分机械设备产业主题ETF联接C
    1983  012600             中银内核驱动股票C
    1984  011043             国泰价值先锋股票C
    1985  012516  国泰中证细分机械设备产业主题ETF联接A
    :param is_explain:
    :param ft:
    :param token:
    """
    params = [
        ('op', 'dy'),
        ('dt', 'kf'),
        ('rs', ''),
        ('gs', '0'),
        ('sc', 'qjzf'),
        ('st', 'desc'),
        ('es', '0'),
        ('qdii', ''),
        ('pi', '1'),
        ('pn', '50000'),
        ('dx', '0'),
    ]
    headers = {
        'Connection': 'keep-alive',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36 Edg/87.0.664.75',
        'Accept': '*/*',
        'Referer': 'http://fund.eastmoney.com/data/fundranking.html',
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
    }
    if ft is not None:
        params.append(('ft', ft))
    url = 'http://fund.eastmoney.com/data/rankhandler.aspx'
    response = requests_obj.get(url, params, headers=headers)
    results = re.findall('\[.*\]', response.text)
    new_results = eval(results[0])
    application_obj = BaseApplication(new_results)
    return exchange_explain(application_obj.deal_fund_list(), is_explain)



# import sharetop as sp

from sharetop import sp_prepare
from sharetop import fund

token = "**"

# fund_list = fund.get_fund_open_rank(token, symbol="股票型")
# sp_obj = sp_prepare.BasicTop(token)
# r = sp_obj.common_exec_func("stock_to_list_data", {"limit": 10, "is_explain": True})
#
# print(r.to_dict("records"))

# r2 = sp_obj.common_exec_func("stock_to_list_data", {"limit": 10, "is_explain": False})

fund_base_list = fund.get_fund_codes(token, ft='zs')

print(fund_base_list)


ship_indicators_url = "/ship/indicators"

# {"year":2015,"wdwt":3922.0,"wdwt_percent":40.8,"cgt":1286.0,"cgt_percent":35.1,"ship_indicators":"1","short_name":"中国"}
ship_indicators_field_dict = {"year": "年份", "wdwt": "万载重吨", "wdwt_percent": "万载重吨所占比重",
                      "cgt": "万修正总吨", "cgt_percent": "万修正总吨百分比", "ship_indicators": "造船指标", "short_name": "机构名称"}



import uuid
import pandas as pd
from typing import List, Union, Dict, Any
from retry import retry
from ..utils import to_numeric, requests_obj, validate_request
from jsonpath import jsonpath
from .config import EastmoneyFundHeaders
from ..common.explain_change import exchange_explain


@validate_request
@retry(tries=3)
@to_numeric
def get_fund_invest_position(token: str,
                             fund_code: str, dates: Union[str, List[str]] = None, is_explain: bool = False
                             ) -> pd.DataFrame:
    """
    获取基金持仓占比数据
    Parameters
    ----------
    fund_code : str
        基金代码
    dates : Union[str, List[str]], optional
        日期或者日期构成的列表
        可选示例如下

        - ``None`` : 最新公开日期
        - ``'2020-01-01'`` : 一个公开持仓日期
        - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
    Returns
    -------
    DataFrame
        基金持仓占比数据
    Examples
    --------
        基金代码    股票代码  股票简称   持仓占比  较上期变化        公开日期
    0  161725  600519  贵州茅台  16.78   1.36  2022-03-31
    1  161725  600809  山西汾酒  15.20   0.52  2022-03-31
    2  161725  000568  泸州老窖  14.57  -0.89  2022-03-31
    3  161725  000858   五粮液  12.83  -1.26  2022-03-31
    4  161725  002304  洋河股份  11.58   0.91  2022-03-31
    5  161725  603369   今世缘   3.75  -0.04  2022-03-31
    6  161725  000799   酒鬼酒   3.40  -0.91  2022-03-31
    7  161725  000596  古井贡酒   3.27  -0.24  2022-03-31
    8  161725  600779   水井坊   2.59  -0.26  2022-03-31
    9  161725  603589   口子窖   2.30  -0.38  2022-03-31

        基金代码    股票代码  股票简称   持仓占比  较上期变化        公开日期
    0   161725  600519  贵州茅台  16.78   1.36  2022-03-31
    1   161725  600809  山西汾酒  15.20   0.52  2022-03-31
    2   161725  000568  泸州老窖  14.57  -0.89  2022-03-31
    3   161725  000858   五粮液  12.83  -1.26  2022-03-31
    4   161725  002304  洋河股份  11.58   0.91  2022-03-31
    5   161725  603369   今世缘   3.75  -0.04  2022-03-31
    6   161725  000799   酒鬼酒   3.40  -0.91  2022-03-31
    7   161725  000596  古井贡酒   3.27  -0.24  2022-03-31
    8   161725  600779   水井坊   2.59  -0.26  2022-03-31
    9   161725  603589   口子窖   2.30  -0.38  2022-03-31
    10  161725  000568  泸州老窖  15.46   0.57  2021-12-31
    11  161725  600519  贵州茅台  15.42   0.63  2021-12-31
    12  161725  600809  山西汾酒  14.68  -1.72  2021-12-31
    13  161725  000858   五粮液  14.09   0.87  2021-12-31
    14  161725  002304  洋河股份  10.67  -1.34  2021-12-31
    15  161725  000799   酒鬼酒   4.31   0.09  2021-12-31
    16  161725  603369   今世缘   3.79   0.81  2021-12-31
    17  161725  000596  古井贡酒   3.51  -0.69  2021-12-31
    18  161725  600779   水井坊   2.85  -0.41  2021-12-31
    19  161725  603589   口子窖   2.68   2.68  2021-12-31
    :param token:
    :param fund_code:
    :param dates:
    :param is_explain:

    """
    columns = {
        'GPDM': '股票代码',
        'GPJC': '股票简称',
        'JZBL': '持仓占比',
        'PCTNVCHG': '较上期变化',
    }
    str_uuid = str(uuid.uuid4()).upper()
    df = pd.DataFrame(columns=columns.values())
    if not isinstance(dates, List):
        dates = [dates]
    if dates is None:
        dates = [None]
    dfs: List[pd.DataFrame] = []
    for date in dates:
        params = [
            ('FCODE', fund_code),
            ('appType', 'ttjj'),
            ('deviceid', str_uuid),
            ('plat', 'Iphone'),
            ('product', 'EFund'),
            ('serverVersion', '6.2.8'),
            ('version', '6.2.8'),
        ]
        if date is not None:
            params.append(('DATE', date))
        url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNInverstPosition'
        json_response = requests_obj.get(url, params, headers=EastmoneyFundHeaders).json()
        stocks = jsonpath(json_response, '$..fundStocks[:]')
        if not stocks:
            continue
        date = json_response['Expansion']
        _df = pd.DataFrame(stocks)
        _df['公开日期'] = date
        _df.insert(0, '基金代码', fund_code)
        dfs.append(_df)
    fields = ['基金代码'] + list(columns.values()) + ['公开日期']
    if not dfs:
        return pd.DataFrame(columns=fields)
    df = pd.concat(dfs, axis=0, ignore_index=True).rename(columns=columns)[fields]
    return exchange_explain(df, is_explain)


@validate_request
def get_fund_public_dates(token: str, fund_code: str) -> Dict[str, Any]:
    """
    获取历史上更新持仓情况的日期列表
    Parameters
    ----------
    fund_code : str
        6 位基金代码
    Returns
    -------
    List[str]
        指定基金公开持仓的日期列表
    Examples
    --------
    ['2021-03-31', '2021-01-08', '2020-12-31', '2020-09-30', '2020-06-30']
    """
    params = (
        ('FCODE', fund_code),
        ('appVersion', '6.3.8'),
        ('deviceid', '3EA024C2-7F22-408B-95E4-383D38160FB3'),
        ('plat', 'Iphone'),
        ('product', 'EFund'),
        ('serverVersion', '6.3.6'),
        ('version', '6.3.8'),
    )
    url = 'https://fundmobapi.eastmoney.com/FundMNewApi/FundMNIVInfoMultiple'
    json_response = requests_obj.get(url, params, headers=EastmoneyFundHeaders).json()
    if json_response['Datas'] is None:
        return {"date_list": []}
    return {"date_list": json_response['Datas']}



import inspect


class Explain:
    get_stock_all_report_dates_fields = {
        "报告日期": "report_date",
        "季报名称": "quarterly_report_title"
    }

    get_stock_market_real_time_data_fields = {
        "股票代码": "stock_code",
        "股票名称": "stock_name",
        "涨跌幅": "percentage_change",
        "最新价": "last_price",
        "最高": "high_price",
        "最低": "low_price",
        "今开": "open_price",
        "涨跌额": "price_change",
        "换手率": "turnover_rate",
        "量比": "volume_ratio",
        "动态市盈率": "price_to_earnings_ratio",
        "成交量": "trading_volume",
        "成交额": "volume",
        "昨日收盘": "previous_close",
        "总市值": "market_cap",
        "流通市值": "free_float_market_cap",
        # "行情ID": "item_id",
        "市场类型": "market_type",
        "更新时间": "update_time",
        "最新交易日": "latest_trading_day"
    }

    get_stock_all_company_quarterly_report_fields = {
        "股票代码": "stock_code",
        "股票简称": "stock_name",
        "公告日期": "public_date",
        "营业收入": "operating_revenue",
        "营业收入同比增长": "yoy_growth_operating_revenue",
        "营业收入季度环比": "qoq_growth_operating_revenue",
        "净利润": "net_profit",
        "净利润同比增长": "yoy_growth_net_profit",
        "净利润季度环比": "qoq_growth_net_profit",
        "每股收益": "eps",
        "每股净资产": "navps",
        "净资产收益率": "roe",
        "销售毛利率": "gross_profit_margin",
        "每股经营现金流量": "eps_operating_cash_flow"
    }

    get_stock_company_report_data_fields = {
        "股票代码": "stock_code",
        "股票名称": "stock_name",
        "板块": "sector",
        "股票类型": "stock_type",
        "最新公告日期": "latest_announcement_date",
        "报告期": "reporting_period",
        "每股收益(元)": "eps",
        "每股收益(扣除)(元)": "eps_diluted",
        "营业总收入(元)": "total_operating_revenue",
        "净利润(元)": "net_profit",
        "净资产收益率(%)": "roe",
        "营业总收入同比增长(%)": "yoy_growth_operating_revenue",
        "净利润同比增长(%)": "yoy_growth_net_profit",
        "每股净资产(元)": "navps",
        "每股经营现金流量(元)": "eps_operating_cash_flow",
        "销售毛利率(%)": "gross_profit_margin",
        "营业总收入季度环比增长(%)": "qoq_growth_operating_revenue",
        "净利润季度环比增长": "qoq_growth_net_profit",
        "利润分配": "profit_distribution",
        "股息率(%)": "dividend_yield",
        "首次公告日期": "initial_announcement_date",
        "年报类型": "annual_report_type"
    }

    get_stock_base_info_fields = {
        "唯一代码": "unique_id",
        "股票代码": "stock_code",
        "市场代码": "market_code",
        "股票名称": "stock_name",
        "公司名称": "org_name",
        "公司英文名称": "org_name_en",
        "统一社会信用代码": "uscc",
        "证券类别": "security_type",
        "上市交易所": "stock_exchange",
        "东财行业": "east_money_industry",
        "证监会行业": "csrc_industry",
        "董事长": "chairman",
        "法人代表": "legal_representative",
        "总经理": "manager",
        "董秘": "board_secretary",
        "证券事务代表": "securities_representative",
        "注册资本(万)": "reg_capital",
        "公司成立日期": "found_date",
        "公司上市日期": "ipo_date",
        "所在省份": "province",
        "所在城市": "city",
        "公司简介": "introduction",
        "经营范围": "business_scope",
        "公司主页": "website",
        "联系电话": "org_tel",
        "联系传真": "org_fax",
        "电子邮件": "email",
        "注册地址": "reg_address",
        "办公地址": "office_address",
        "员工人数": "emp_num",
        "管理人员人数": "manager_num",
        "律师事务所": "law_office",
        "会计师事务所": "accounting_firm",
        "交易所代码": "exchange_code"
    }

    get_stock_real_time_data_fields = {
        "股票名称": "stock_name",
        "股票代码": "stock_code",
        "最新价": "latest_price",
        "最高价": "highest_price",
        "最低价": "lowest_price",
        "开盘价": "open_price",
        "成交量(手)": "trading_volume",
        "成交额(元)": "volume",
        "外盘(手)": "foreign_volume(lots)",
        "昨收": "previous_close",
        "均价": "average_price",
        "总股本": "total_share_capital",
        "流通股本": "floating_shares",
        "每股净资产": "navps",
        "每股收益": "eps",
        "总市值": "market_cap",
        "流通市值": "free_float_market_cap",
        "内盘(手)": "domestic_volume(lots)",
        "市盈率(TTM)": "ttm",
        "市净率": "pb_ratio",
        "涨跌值": "price_change",
        "涨跌幅(%)": "percentage_change",
        "振幅(%)": "amplitude"
    }

    get_stock_kline_data_fields = {
        "股票名称": "stock_name",
        "股票代码": "stock_code",
        "日期": "date",
        "开盘": "opening_price",
        "收盘": "closing_price",
        "最高": "highest_price",
        "最低": "lowest_price",
        "成交量": "trading_volume",
        "成交额": "volume",
        "振幅": "amplitude",
        "涨跌幅": "percentage_change",
        "涨跌额": "price_change",
        "换手率": "turnover_rate",
        "盘后量": "after_hours_volume",
        "盘后额": "after_hours_value"
    }

    get_stock_history_capital_fields = {
        "股票名称": "stock_name",
        "股票代码": "stock_code",
        "日期": "date",
        "主力净流入": "net_main_inflow",
        "小单净流入": "small_order_net_inflow",
        "中单净流入": "medium_order_net_inflow",
        "大单净流入": "large_order_net_inflow",
        "超大单净流入": "super_large_order_net_inflow",
        "主力净流入占比": "main_net_inflow_ratio",
        "小单流入净占比": "smail_order_net_inflow_ratio",
        "中单流入净占比": "medium_order_net_inflow_ratio",
        "大单流入净占比": "large_order_net_inflow_ratio",
        "超大单流入净占比": "super_large_order_net_inflow_ratio",
        "收盘价": "closing_price",
        "涨跌幅": "percentage_change"
    }

    get_stock_real_time_daily_capital_fields = {
        "股票名称": "stock_name",
        "股票代码": "stock_code",
        "时间": "time",
        "主力净流入": "net_main_inflow",
        "小单净流入": "small_order_net_inflow",
        "中单净流入": "medium_order_net_inflow",
        "大单净流入": "large_order_net_inflow",
        "超大单净流入": "super_large_order_net_inflow"
    }

    get_stock_real_time_sum_capital_fields = {
        "今日主力净流入": "net_main_inflow_today",
        "超大单流入": "super_large_inflow",
        "超大单流出": "super_large_outflow",
        "今日超大单净流入": "net_super_large_inflow_today",
        "超大单净比": "net_super_large_ratio",
        "大单流入": "large_inflow",
        "大单流出": "large_outflow",
        "今日大单净流入": "net_large_inflow_today",
        "大单净比": "net_large_ratio",
        "中单流入": "medium_inflow",
        "中单流出": "medium_outflow",
        "今日中单净流入": "net_medium_inflow_today",
        "中单净比": "net_medium_ratio",
        "小单流入": "small_inflow",
        "小单流出": "small_outflow",
        "今日小单净流入": "net_small_inflow_today",
        "小单净比": "net_small_ratio",
        "主力净比": "net_main_ratio",
        "20日主力净流入": "net_main_inflow_20_days",
        "20日超大单净流入": "net_super_large_inflow_20_days",
        "20日大单净流入": "net_large_inflow_20_days",
        "20日中单净流入": "net_medium_inflow_20_days",
        "20日小单净流入": "net_small_inflow_20_days",
        "5日主力净流入": "net_main_inflow_5_days",
        "5日超大单净流入": "net_super_large_inflow_5_days",
        "5日大单净流入": "net_large_inflow_5_days",
        "5日中单净流入": "net_medium_inflow_5_days",
        "5日小单净流入": "net_small_inflow_5_days"
    }

    get_stock_real_time_sector_capital_fields = {
        "今日涨跌幅(%)": "percentage_change_today",
        "行业": "industry",
        "今日主力净流入额": "net_main_inflow_amount_today",
        "今日超大单净流入额": "net_super_large_inflow_amount_today",
        "今日超大单流入净占比(%)": "net_super_large_inflow_ratio_today",
        "今日大单净流入额": "net_large_inflow_amount_today",
        "今日大单流入净占比(%)": "net_large_inflow_ratio_today",
        "今日中单净流入额": "net_medium_inflow_amount_today",
        "今日中单流入净占比(%)": "net_medium_inflow_ratio_today",
        "今日小单净流入额": "net_small_inflow_ratio_today",
        "今日小单流入净占比(%)": "net_small_inflow_ratio_today",
        "今日主力流入净占比(%)": "net_main_inflow_ratio_today",
        "股票名称": "stock_name",
        "股票代码": "stock_code"
    }

    get_stock_dragon_tiger_list_fields = {
        "股票名称": "stock_name",
        "股票代码": "stock_code",
        "上榜日期": "listing_date",
        "解读": "interpretation",
        "收盘价": "closing_price",
        "涨跌幅": "percentage_change",
        "换手率": "turnover_rate",
        "龙虎榜净买额": "net_buying_amount_on_dtl",
        "龙虎榜买入额": "buying_amount_on_dtl",
        "龙虎榜卖出额": "selling_amount_on_dtl",
        "龙虎榜成交额": "transaction_amount_on_dtl",
        "市场总成交额": "total_market_turnover",
        "净买额占总成交比": "net_buy_ratio",
        "成交额占总成交比": "turnover_ratio",
        "流通市值": "free_float_market_cap",
        "上榜原因": "reason_listing"
    }

    get_fund_codes_fields = {
        "基金代码": "fund_code",
        "基金名称": "fund_name",
        "自成立以来收益(%)": "cumulative_return_since_inception",
        "成立时间": "inception_date",
        "净值时间": "nav_date",
        "单位净值": "nav_per_unit",
        "累计净值": "cumulative_net_asset_value"
    }

    get_fund_open_rank_fields = {
        "序号": "number",
        "基金代码": "fund_code",
        "基金简称": "fund_name",
        "日期": "date",
        "单位净值": "nav_per_unit",
        "累计净值": "cumulative_net_asset_value",
        "日增长率": "daily_growth_rate",
        "近1周": "last_one_week",
        "近1月": "last_one_month",
        "近3月": "last_three_month",
        "近6月": "last_six_month",
        "近1年": "last_one_year",
        "近2年": "last_two_year",
        "近3年": "last_three_year",
        "今年来": "year_to_day",
        "成立来": "since_inception",
        "手续费": "handling_fee"
    }

    get_fund_exchange_rank_fields = {
        '序号': 'number',
        "基金代码": "fund_code",
        "基金简称": "fund_name",
        "类型": "fund_type",
        "日期": "date",
        "单位净值": "nav_per_unit",
        "累计净值": "cumulative_net_asset_value",
        "近1周": "last_one_week",
        "近1月": "last_one_month",
        "近3月": "last_three_month",
        "近6月": "last_six_month",
        "近1年": "last_one_year",
        "近2年": "last_two_year",
        "近3年": "last_three_year",
        "今年来": "year_to_day",
        "成立来": "since_inception",
        "成立日期": "inception_date"
    }

    get_fund_money_rank_fields = {
        "序号": "number",
        "近1月": "last_one_month",
        "近3月": "last_three_month",
        "近6月": "last_six_month",
        "近1年": "last_one_year",
        "近2年": "last_two_year",
        "近3年": "last_three_year",
        "近5年": "last_five_year",
        "基金代码": "fund_code",
        "基金简称": "fund_name",
        "日期": "date",
        "万份收益": "income_yield",
        "年化收益率7日": "7_day_annualized_yield",
        "年化收益率14日": "14_day_annualized_yield",
        "年化收益率28日": "28_day_annualized_yield",
        "今年来": "year_to_day",
        "成立来": "since_inception"
    }

    get_fund_hk_rank_fields = {
        "序号": "number",
        "基金代码": "fund_code",
        "基金简称": "fund_name",
        "币种": "currency",
        "日期": "date",
        "单位净值": "nav_per_unit",
        "日增长率": "daily_growth_rate",
        "近1周": "last_one_week",
        "近1月": "last_one_month",
        "近3月": "last_three_month",
        "近6月": "last_six_month",
        "近1年": "last_one_year",
        "近2年": "last_two_year",
        "近3年": "last_three_year",
        "今年来": "year_to_day",
        "成立来": "since_inception",
        "可购买": "available_purchase",
        "香港基金代码": "hong_kong_fund_code"
    }

    get_fund_base_info_fields = {
        "基金代码": "fund_code",
        "基金简称": "fund_name",
        "涨跌幅": "percentage_change",
        "最新净值": "latest_nav",
        "成立日期": "inception_date",
        "最新评级": "latest_rating",
        "基金公司": "fund_company",
        "净值更新日期": "nav_update_date",
        "基金规模": "fund_size",
        "基金规模更新时间": "fund_size_update_date",
        "简介": "introduction"
    }

    get_fund_history_price_fields = {
        "日期": "date",
        "单位净值": "nav_per_unit",
        "累计净值": "cumulative_net_asset_value",
        "涨跌幅": "percentage_change"
    }

    get_fund_industry_distribution_fields = {
        "基金代码": "fund_code",
        "行业名称": "industry_name",
        "持仓比例": "position_ratio",
        "公布日期": "announcement_date",
        "市值": "market_cap"
    }

    get_fund_invest_position_fields = {
        "基金代码": "fund_code",
        "股票代码": "stock_code",
        "股票简称": "stock_name",
        "持仓占比": "weight",
        "较上期变化": "change",
        "公开日期": "pub_date"
    }

    get_fund_period_change_fields = {
        "基金代码": "fund_code",
        "收益率": "yield",
        "同类平均": "category_average",
        "同类排行": "category_ranking",
        "同类总数": "total_number_categories",
        "时间段": "time_period"
    }

    get_fund_types_percentage_fields = {
        "基金代码": "fund_code",
        "股票比重": "stock_weight",
        "债券比重": "bond_weight",
        "现金比重": "cash_weight",
        "总规模(亿元)": "total_size",
        "其他比重": "other_proportions"
    }

    get_bond_base_info_list_fields = {
        "债券代码": "bond_code",
        "债券名称": "bond_name",
        "正股代码": "equity_code",
        "正股名称": "equity_name",
        "债券评级": "bond_rating",
        "申购日期": "subscription_date",
        "发行规模(亿)": "issuance_size",
        "网上发行中签率(%)": "online_subscription_winning_rate",
        "上市日期": "listing_date",
        "到期日期": "maturity_date",
        "期限(年)": "term",
        "利率说明": "interest_rate_description"
    }

    get_bond_realtime_quotes_fields = {
        "债券代码": "bond_code",
        "债券名称": "bond_name",
        "涨跌幅": "percentage_change",
        "最新价": "last_price",
        "最高": "high_price",
        "最低": "low_price",
        "今开": "open_price",
        "涨跌额": "price_change",
        "换手率": "turnover_rate",
        "量比": "volume_ratio",
        "动态市盈率": "price_to_earnings_ratio",
        "成交量": "trading_volume",
        "成交额": "volume",
        "昨日收盘": "previous_close",
        "总市值": "market_cap",
        "流通市值": "free_float_market_cap",
        "行情ID": "item_id",
        "市场类型": "market_type",
        "更新时间": "update_time",
        "最新交易日": "latest_trading_day"
    }

    get_bond_base_info_fields = {
        "债券代码": "bond_code",
        "债券名称": "bond_name",
        "正股代码": "equity_code",
        "正股名称": "equity_name",
        "债券评级": "bond_rating",
        "申购日期": "subscription_date",
        "发行规模(亿)": "issuance_size",
        "网上发行中签率(%)": "online_subscription_winning_rate",
        "上市日期": "listing_date",
        "到期日期": "maturity_date",
        "期限(年)": "term",
        "利率说明": "interest_rate_description"
    }

    get_future_all_realtime_quotes_fields = {
        "期货代码": "futures_code",
        "期货名称": "futures_name",
        "涨跌幅": "percentage_change",
        "最新价": "last_price",
        "最高": "high_price",
        "最低": "low_price",
        "今开": "open_price",
        "涨跌额": "price_change",
        "换手率": "turnover_rate",
        "量比": "volume_ratio",
        "动态市盈率": "price_to_earnings_ratio",
        "成交量": "trading_volume",
        "成交额": "volume",
        "昨日收盘": "previous_close",
        "总市值": "market_cap",
        "流通市值": "free_float_market_cap",
        "行情ID": "item_id",
        "市场类型": "market_type",
        "更新时间": "update_time",
        "最新交易日": "latest_trading_day"
    }

    get_future_deal_detail_fields = {
        "期货代码": "futures_code",
        "期货名称": "futures_name",
        "时间": "time",
        "昨收": "previous_close",
        "成交价": "transaction_price",
        "成交量": "trading_volume",
        "单数": "lot_size"
    }

    get_future_history_data_fields = {
        "期货代码": "futures_code",
        "期货名称": "futures_name",
        "日期": "date",
        "开盘": "opening_price",
        "收盘": "closing_price",
        "最高": "highest_price",
        "最低": "low_price",
        "成交量": "trading_volume",
        "成交额": "volume",
        "振幅": "amplitude",
        "涨跌幅": "percentage_change",
        "涨跌额": "price_change",
        "换手率": "turnover_rate",
        "盘后量": "after_hours_volume",
        "盘后额": "after_hours_value"
    }

    get_country_list_fields = {
        "国家简称": "country_abbreviation",
        "国家全称": "country_full_name",
        "英文简称": "english_abbreviation",
        "英文全称": "english_full_name",
        "土地面积(万平方公里)": "land_area",
        "所属洲": "continent",
        "机构类型": "institution_type",
        "首都(中文)": "capital_ch",
        "首都(英文)": "capital_en",
        "所属组织": "organization"
    }

    get_oil_reserves_fields = {
        "年份": "year",
        "储量(十亿桶)": "reserves",
        "总储量占比": "total_reserves_ratio",
        "储产比": "rpr",
        "产品名称": "product_name",
        "机构名称": "institution_name"
    }

    get_oil_products_fields = {
        "年份": "year",
        "产量(千桶/天)": "production",
        "总产量占比": "total_production_proportion",
        "产品名称": "product_name",
        "机构名称": "institution_name"
    }

    get_oil_consumption_fields = {
        "年份": "year",
        "产量(千桶/天)": "production",
        "总产量占比": "total_production_proportion",
        "产品名称": "product_name",
        "机构名称": "institution_name"
    }

    get_oil_refinerythroughput_fields = {
        "年份": "year",
        "加工量(千桶/天)": "processing_volume",
        "总产量占比": "total_production_proportion",
        "产品名称": "product_name",
        "机构名称": "institution_name"
    }

    get_oil_refinerycapacity_fields = {
        "年份": "year",
        "产能(千桶/天)": "production_capacity",
        "总产能占比": "total_production_capacity_ratio",
        "产品名称": "product_name",
        "机构名称": "institution_name"
    }

    get_oil_crudeoilpricehistory_fields = {
        "年份": "year",
        "价格(当年真实价格)": "current_year_real_price",
        "价格(按2022年美国消费者物价指数平减)": "2022_price",
        "产品名称": "product_name"
    }

    get_pig_fcr_fields = {
        "发布时间": "pub_date",
        "生猪(外三元)(元/公斤)": "pork_duroc_price",
        "猪粮比": "fcr"
    }

    get_ship_indicators_fields = {
        "年份": "year",
        "万载重吨": "wdwt",
        "万载重吨所占比重": "wdwt_percent",
        "万修正总吨": "cgt",
        "万修正总吨百分比": "cgt_percent",
        "造船指标": "ship_indicators",
        "机构名称": "short_name"
    }

    get_car_sales_fields = {
        "排名": "rank",
        "车系": "car_series",
        "经销商最低价格": "dealer_min_price",
        "经销商最高价格": "dealer_max_price",
        "厂商最高价格": "maker_max_price",
        "厂商最低价格": "maker_min_price",
        "总品牌": "brand_name",
        "子品牌": "sub_brand_name",
        "销量": "sales_volume",
        "汽车类型": "car_type",
        "公布时间": "pub_date",
        "数据源": "source"
    }

    car_to_kline_data_fields = {
        "排名": "rank",
        "车系": "car_series",
        "经销商最低价格": "dealer_min_price",
        "经销商最高价格": "dealer_max_price",
        "厂商最高价格": "maker_max_price",
        "厂商最低价格": "maker_min_price",
        "总品牌": "brand_name",
        "子品牌": "sub_brand_name",
        "销量": "sales_volume",
        "汽车类型": "car_type",
        "公布时间": "pub_date",
        "数据源": "source"
    }

    get_logistics_index_fields = {
        "指标值": "index_value",
        "指数类型": "index_type",
        "指数日期": "index_date"
    }

    get_fund_real_time_god_fields = {
        "基金名称": "fund_name",
        "基金代码": "fund_code",
        "净值估算": "net_value_estimation",
        "涨跌值": "price_change",
        "估算涨跌幅": "percentage_change_estimation",
        "更新时间": "update_time"
    }

    bond_yield_to_real_time_fields = {
        "国债名称": "bond_name",
        "开盘价格": "open_price",
        "昨收": "previous_close",
        "当前价格": "now_price",
        "最高": "high_price",
        "最低": "low_price",
        "涨跌幅(%)": "percentage_change",
        "涨跌额": "price_change",
        "更新时间": "update_time"
    }

    bond_yield_to_kline_data_fields = {
        "国债代码": "bond_code",
        "开盘价格": "open_price",
        "收盘价格": "close_price",
        "最高": "high_price",
        "最低": "low_price",
        "涨跌幅(%)": "percentage_change",
        "涨跌额": "price_change",
        "更新时间": "day"
    }

    pig_to_warning_kline_data_fields = {
        "级别": "grade",
        "涨跌": "status",
        "公布时间": "pub_date"
    }

    pig_to_fcr_kline_data_fields = {
        "发布时间": "pub_date",
        "生猪(外三元)(元/公斤)": "pig_price_one",
        "猪粮比": "pig_grain_ratio"
    }

    stock_to_list_data_fields = {'ts代码': 'ts_code', '股票代码': 'code', '市场代码': 'mk_code', '股票名称': 'name',
                                  '股票全称': 'fullname', '英文全称': 'enname', '拼音缩写': 'ren_ping',
                                  '市场类别': 'market', '交易所代码': 'exchange_code', '交易货币': 'curr_type',
                                  '上市状态': 'list_status', '上市日期': 'list_date', '退市日期': 'delist_date',
                                  '成立日期': 'fund_date', '是否沪深港通标的': 'is_hs', '地域': 'area',
                                  '所属行业': 'industry', '曾用名列表': 'former_name', '发行方式': 'audit_type'}

    stock_to_trade_date_list_fields = {
        "交易时间": "cal_date"
    }

    stock_to_index_list_fields = {
        'ts代码': 'ts_code', '指数代码': 'code', '市场代码': 'mk_code', '指数名称': 'name'
    }

    stock_to_intl_index_list_fields = {
        'ts代码': 'ts_code', '指数代码': 'code', '指数区域': 'region', '指数名称': 'name'
    }

    stock_to_index_stock_composition_fields = {
        'ts代码': 'ts_code', '股票代码': 'code', '股票所在城市': 'region', '股票名称': 'name', "指数类型": "index_type",
        "所属行业": "industry"
    }

    stock_to_stock_repurchase_fields = {
        'ts代码': 'ts_code',
        '股票名称': 'name',
        '计划回购价格区间(元)': 'repurpricecap',
        '计划回购股票数量区间最小值(股)': 'repurnumlower',
        "计划回购股票数量区间最大值(股)": "repurnumcap",
        "占公告前一日总股本比例最小值(%)": "zszxx",
        "占公告前一日总股本比例最大值(%)": "zszsx",
        "计划回购金额区间最小值(元)": "repuramountlower",
        "计划回购金额区间最大值(元)": "repuramountlimit",
        "回购起始时间": "repurstartdate",
        "实施进度": "repurprogress",
        "已回购股份价格区间最小值(元)": "repurpricelower1",
        "已回购股份价格区间最大值(元)": "repurpricecap1",
        "已回购股票数量(股)": "repurnum",
        "已回购金额(元)": "repuramount",
        "最新公告日期": "updatedate"
    }

    stock_to_stock_bonus_fields = {
        'ts代码': 'ts_code',
        '股票名称': 'name',
        '报告期': 'report_date',
        '业绩披露日期': 'plan_notice_date',
        "送转总比例": "bonus_it_ratio",
        "送股比例": "bonus_ratio",
        "转股比例": "it_ratio",
        "现金分红比例": "pretax_bonus_rmb",
        "股息率": "divident_ratio",
        "每股收益(元)": "basic_eps",
        "每股净资产(元)": "bvps",
        "每股公积金(元)": "per_capital_reserve",
        "每股未分配利润(元)": "per_unassign_profit",
        "净利润同比增长": "pnp_yoy_ratio",
        "总股本(亿)": "total_shares",
        "预案公告日": "publish_date",
        "股权登记日": "equity_record_date",
        "除权除息日": "ex_dividend_date",
        "方案进度": "assign_progress",
        "最新公告日期": "最新公告日期"
    }

    news_to_normal_fields = {
        "发布时间": "display_time",
        "新闻链接": "source_url",
        "标题": "title",
        "新闻内容": "content",
        "新闻简述": "content_short",
        "新闻标签": "tags"
    }


def exchange_explain_one(df, exchange_fields, is_explain):
    if not is_explain:
        df.rename(columns=exchange_fields, inplace=True)
    return df


def exchange_explain(df, is_explain):
    frame = inspect.currentframe().f_back
    func_name_fields = frame.f_code.co_name
    exchange_fields = getattr(Explain, f"{func_name_fields}_fields")
    if not is_explain:
        if isinstance(df, dict):
            return {k: exchange_explain_one(v, exchange_fields, is_explain) for k, v in df.items()}
        df.rename(columns=exchange_fields, inplace=True)
    return df


def exchange_explain_new(df, is_explain):
    frame = inspect.currentframe().f_back
    func_name_fields = frame.f_code.co_name
    exchange_fields = getattr(Explain, f"{func_name_fields}_fields")
    exchange_fields = {v: k for k, v in exchange_fields.items()}
    if is_explain:
        if isinstance(df, dict):
            return {k: exchange_explain_one(v, exchange_fields, is_explain) for k, v in df.items()}
        df.rename(columns=exchange_fields, inplace=True)
    return df



__title__ = 'sharetop'
__version__ = '6.16.21'
__author__ = 'nrliangxy'
__url__ = 'https://github.com/nrliangxy/sharetop'
__author_email__ = 'nrliangxy@foxmail.com'
__keywords__ = ['data', 'finance', 'quant', 'stock', 'fund', 'futures', 'share']
__description__ = 'Provide data related to the economic market'
__project_urls__ = {
    'Source': 'https://github.com/nrliangxy/sharetop',
}



import pandas as pd
import math
from typing import List
from jsonpath import jsonpath
from datetime import datetime
from ..core.common.config import MARKET_NUMBER_DICT


class BaseApplication:
    def __init__(self, json_data, *args, **kwargs):
        self.json_data = json_data

    def parse_json(self, json_par):
        json_r: List[str] = jsonpath(self.json_data, json_par)
        return json_r

    def deal_k_data(self, columns, quote_id, json_par='$..klines[:]'):
        k_lines = self.parse_json(json_par)
        if not k_lines:
            columns.insert(0, '代码')
            columns.insert(0, '名称')
            return pd.DataFrame(columns=columns)
        rows = [kline.split(',') for kline in k_lines]
        name = self.json_data['data']['name']
        code = quote_id.split('.')[-1]
        df = pd.DataFrame(rows, columns=columns)
        df.insert(0, '代码', code)
        df.insert(0, '名称', name)
        return df

    def deal_fields(self, data, fields_list):
        f59 = data['f59']
        for _ in fields_list:
            item = data[_] / math.pow(10, f59)
            data[_] = item
        return data

    def deal_price_fields(self, data, fields_list):
        f152 = data['f152']
        for _ in fields_list:
            item = data[_] / math.pow(10, f152)
            data[_] = item
        return data

    def deal_real_time_data(self, columns, fields_k_v):
        data = self.json_data['data']
        if not data:
            columns.insert(0, '代码')
            columns.insert(0, '名称')
            return pd.DataFrame(columns=columns)
        deal_real_time_price = ["f170"]
        deal_fields_list = ["f43", "f169", "f44", "f45", "f46", "f60", "f71", "f164", "f167", "f171"]
        data = self.deal_fields(data, deal_fields_list)
        data = self.deal_price_fields(data, deal_real_time_price)
        r = {v: data.get(k) for k, v in fields_k_v.items()}
        return pd.DataFrame([r])

    def deal_market_realtime(self, columns):
        df = pd.DataFrame(self.json_data['data']['diff'])
        df = df.rename(columns=columns)
        df: pd.DataFrame = df[columns.values()]
        # df['行情ID'] = df['市场编号'].astype(str) + '.' + df['代码'].astype(str)
        df['市场类型'] = df['市场编号'].astype(str).apply(lambda x: MARKET_NUMBER_DICT.get(x))
        df['更新时间'] = df['更新时间戳'].apply(lambda x: str(datetime.fromtimestamp(x)))
        df['最新交易日'] = pd.to_datetime(df['最新交易日'], format='%Y%m%d').astype(str)
        tmp = df['最新交易日']
        del df['最新交易日']
        df['最新交易日'] = tmp
        del df['更新时间戳']
        return df

    def deal_quarterly_report(self, columns):
        df = pd.DataFrame(self.json_data['result']['data'])
        df = df.drop(['TRADE_MARKET_CODE', 'SECURITY_TYPE_CODE', 'PAYYEAR', 'PUBLISHNAME', 'ORG_CODE', 'TRADE_MARKET_ZJG',
                 'ISNEW', 'DATAYEAR', 'DATEMMDD', 'EITIME', 'SECUCODE', 'QDATE'], axis=1)
        df = df.rename(columns=columns)
        return df

    def deal_bill(self, columns, quote_id, json_par='$..klines[:]'):
        klines: List[str] = self.parse_json(json_par)
        if not klines:
            columns.insert(0, '代码')
            columns.insert(0, '名称')
            return pd.DataFrame(columns=columns)
        rows = [kline.split(',') for kline in klines]
        name = jsonpath(self.json_data, '$..name')[0]
        code = quote_id.split('.')[-1]
        df = pd.DataFrame(rows, columns=columns)
        df.insert(0, '代码', code)
        df.insert(0, '名称', name)
        return df

    def deal_fund_list(self):
        def fund_data(item):
            r1_list = item.split(",")
            code = r1_list[0]
            name = r1_list[1]
            base_yield = r1_list[3]
            funded_time = r1_list[12]
            now_time = r1_list[9]
            unit_value = r1_list[10]
            accumulate_value = r1_list[11]
            value_list = [code, name, base_yield, funded_time, now_time, unit_value, accumulate_value]
            key_list = ["基金代码", "基金名称", "自成立以来收益(%)", "成立时间", "净值时间", "单位净值", "累计净值"]
            return dict(zip(key_list, value_list))
        return pd.DataFrame(list(map(fund_data, self.json_data)))



from .oil_detail import (
    get_oil_reserves,
    get_oil_products,
    get_oil_consumption,
    get_oil_refinerythroughput,
    get_oil_refinerycapacity,
    get_oil_crudeoilpricehistory
)


__all__ = [
    'get_oil_reserves',
    'get_oil_products',
    'get_oil_consumption',
    'get_oil_refinerythroughput',
    'get_oil_refinerycapacity',
    'get_oil_crudeoilpricehistory'
]



import time
import pandas as pd
import requests
from py_mini_racer import py_mini_racer
from .config import js_str


def bond_treasure_issue_cninfo(
        start_date: str = "20210910", end_date: str = "20211109"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-国债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 结束统计数据
    :type end_date: str
    :return: 国债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1120"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F009D": "缴款日",
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F004D": "发行起始日",
            "F003D": "发行终止日",
            "F008N": "单位面值",
            "SECCODE": "债券代码",
            "F007N": "发行价格",
            "F006N": "计划发行总量",
            "F005N": "实际发行总量",
            "F028N": "增发次数",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F002V": "交易市场",
            "F013V": "发行方式",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "发行起始日",
            "发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行价格",
            "单位面值",
            "缴款日",
            "增发次数",
            "交易市场",
            "发行方式",
            "发行对象",
            "公告日期",
            "债券名称",
        ]
    ]
    temp_df["发行起始日"] = pd.to_datetime(temp_df["发行起始日"]).dt.date
    temp_df["发行终止日"] = pd.to_datetime(temp_df["发行终止日"]).dt.date
    temp_df["缴款日"] = pd.to_datetime(temp_df["缴款日"]).dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"]).dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"])
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"])
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"])
    temp_df["单位面值"] = pd.to_numeric(temp_df["单位面值"])
    temp_df["增发次数"] = pd.to_numeric(temp_df["增发次数"])
    return temp_df


def bond_local_government_issue_cninfo(
        start_date: str = "20210911", end_date: str = "20211110"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-地方债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 开始统计时间
    :type end_date: str
    :return: 地方债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1121"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    print("data_json======:", data_json)
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F009D": "缴款日",
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F004D": "发行起始日",
            "F003D": "发行终止日",
            "F008N": "单位面值",
            "SECCODE": "债券代码",
            "F007N": "发行价格",
            "F006N": "计划发行总量",
            "F005N": "实际发行总量",
            "F028N": "增发次数",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F002V": "交易市场",
            "F013V": "发行方式",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "发行起始日",
            "发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行价格",
            "单位面值",
            "缴款日",
            "增发次数",
            "交易市场",
            "发行方式",
            "发行对象",
            "公告日期",
            "债券名称",
        ]
    ]
    temp_df["发行起始日"] = pd.to_datetime(temp_df["发行起始日"]).dt.date
    temp_df["发行终止日"] = pd.to_datetime(temp_df["发行终止日"]).dt.date
    temp_df["缴款日"] = pd.to_datetime(temp_df["缴款日"]).dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"]).dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"])
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"])
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"])
    temp_df["单位面值"] = pd.to_numeric(temp_df["单位面值"])
    temp_df["增发次数"] = pd.to_numeric(temp_df["增发次数"])
    return temp_df


def bond_corporate_issue_cninfo(
        start_date: str = "20210911", end_date: str = "20211110"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-企业债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 开始统计时间
    :type end_date: str
    :return: 企业债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1122"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "SECNAME": "债券简称",
            "DECLAREDATE": "公告日期",
            "F004D": "交易所网上发行终止日",
            "F003D": "交易所网上发行起始日",
            "F008N": "发行面值",
            "SECCODE": "债券代码",
            "F007N": "发行价格",
            "F006N": "实际发行总量",
            "F005N": "计划发行总量",
            "F022N": "最小认购单位",
            "F017V": "承销方式",
            "F052N": "最低认购额",
            "F015V": "发行范围",
            "BONDNAME": "债券名称",
            "F014V": "发行对象",
            "F013V": "发行方式",
            "F023V": "募资用途说明",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "债券代码",
            "债券简称",
            "公告日期",
            "交易所网上发行起始日",
            "交易所网上发行终止日",
            "计划发行总量",
            "实际发行总量",
            "发行面值",
            "发行价格",
            "发行方式",
            "发行对象",
            "发行范围",
            "承销方式",
            "最小认购单位",
            "募资用途说明",
            "最低认购额",
            "债券名称",
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"]).dt.date
    temp_df["交易所网上发行起始日"] = pd.to_datetime(temp_df["交易所网上发行起始日"]).dt.date
    temp_df["交易所网上发行终止日"] = pd.to_datetime(temp_df["交易所网上发行终止日"]).dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"])
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"])
    temp_df["发行面值"] = pd.to_numeric(temp_df["发行面值"])
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"])
    temp_df["最小认购单位"] = pd.to_numeric(temp_df["最小认购单位"])
    temp_df["最低认购额"] = pd.to_numeric(temp_df["最低认购额"])
    return temp_df

def bond_cov_issue_cninfo(
        start_date: str = "20210913", end_date: str = "20211112"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-可转债发行
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 开始统计时间
    :type end_date: str
    :return: 可转债发行
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1123"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            'F029D': '发行起始日',
            'SECNAME': '债券简称',
            'F027D': '转股开始日期',
            'F003D': '发行终止日',
            'F007N': '发行面值',
            'F053D': '转股终止日期',
            'F005N': '计划发行总量',
            'F051D': '网上申购日期',
            'F026N': '初始转股价格',
            'F066N': '网上申购数量下限',
            'F052N': '发行价格',
            'BONDNAME': '债券名称',
            'F014V': '发行对象',
            'F002V': '交易市场',
            'F032V': '网上申购简称',
            'F086V': '转股代码',
            'DECLAREDATE': '公告日期',
            'F028D': '债权登记日',
            'F004D': '优先申购日',
            'F068D': '网上申购中签结果公告日及退款日',
            'F054D': '优先申购缴款日',
            'F008N': '网上申购数量上限',
            'SECCODE': '债券代码',
            'F006N': '实际发行总量',
            'F067N': '网上申购单位',
            'F065N': '配售价格',
            'F017V': '承销方式',
            'F015V': '发行范围',
            'F013V': '发行方式',
            'F021V': '募资用途说明',
            'F031V': '网上申购代码'
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            '债券代码',
            '债券简称',
            '公告日期',
            '发行起始日',
            '发行终止日',
            '计划发行总量',
            '实际发行总量',
            '发行面值',
            '发行价格',
            '发行方式',
            '发行对象',
            '发行范围',
            '承销方式',
            '募资用途说明',
            '初始转股价格',
            '转股开始日期',
            '转股终止日期',
            '网上申购日期',
            '网上申购代码',
            '网上申购简称',
            '网上申购数量上限',
            '网上申购数量下限',
            '网上申购单位',
            '网上申购中签结果公告日及退款日',
            '优先申购日',
            '配售价格',
            '债权登记日',
            '优先申购缴款日',
            '转股代码',
            '交易市场',
            '债券名称',
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"]).dt.date
    temp_df["发行起始日"] = pd.to_datetime(temp_df["发行起始日"]).dt.date
    temp_df["发行终止日"] = pd.to_datetime(temp_df["发行终止日"]).dt.date
    temp_df["转股开始日期"] = pd.to_datetime(temp_df["转股开始日期"]).dt.date
    temp_df["转股终止日期"] = pd.to_datetime(temp_df["转股终止日期"]).dt.date
    temp_df["转股终止日期"] = pd.to_datetime(temp_df["转股终止日期"]).dt.date
    temp_df["网上申购日期"] = pd.to_datetime(temp_df["网上申购日期"]).dt.date
    temp_df["网上申购中签结果公告日及退款日"] = pd.to_datetime(temp_df["网上申购中签结果公告日及退款日"]).dt.date
    temp_df["债权登记日"] = pd.to_datetime(temp_df["债权登记日"]).dt.date
    temp_df["优先申购日"] = pd.to_datetime(temp_df["优先申购日"]).dt.date
    temp_df["优先申购缴款日"] = pd.to_datetime(temp_df["优先申购缴款日"]).dt.date
    temp_df["计划发行总量"] = pd.to_numeric(temp_df["计划发行总量"])
    temp_df["实际发行总量"] = pd.to_numeric(temp_df["实际发行总量"])
    temp_df["发行面值"] = pd.to_numeric(temp_df["发行面值"])
    temp_df["发行价格"] = pd.to_numeric(temp_df["发行价格"])
    temp_df["初始转股价格"] = pd.to_numeric(temp_df["初始转股价格"])
    temp_df["网上申购数量上限"] = pd.to_numeric(temp_df["网上申购数量上限"])
    temp_df["网上申购数量下限"] = pd.to_numeric(temp_df["网上申购数量下限"])
    temp_df["网上申购单位"] = pd.to_numeric(temp_df["网上申购单位"])
    temp_df["配售价格"] = pd.to_numeric(temp_df["配售价格"])
    return temp_df


def bond_cov_stock_issue_cninfo() -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-债券报表-债券发行-可转债转股
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :return: 可转债转股
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1124"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            'F003N': '转股价格',
            'SECNAME': '债券简称',
            'DECLAREDATE': '公告日期',
            'F005D': '自愿转换期终止日',
            'F004D': '自愿转换期起始日',
            'F017V': '标的股票',
            'BONDNAME': '债券名称',
            'F002V': '转股简称',
            'F001V': '转股代码',
            'SECCODE': '债券代码',
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            '债券代码',
            '债券简称',
            '公告日期',
            '转股代码',
            '转股简称',
            '转股价格',
            '自愿转换期起始日',
            '自愿转换期终止日',
            '标的股票',
            '债券名称',
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"]).dt.date
    temp_df["自愿转换期起始日"] = pd.to_datetime(temp_df["自愿转换期起始日"]).dt.date
    temp_df["自愿转换期终止日"] = pd.to_datetime(temp_df["自愿转换期终止日"]).dt.date
    temp_df["转股价格"] = pd.to_numeric(temp_df["转股价格"])
    return temp_df



import requests
import threading
import copy
import pandas as pd
import json
from typing import List, Union
from ...crawl.settings import fund_valuation_url_list
from ..common.explain_change import exchange_explain

data_source = []


def get_fund_price(fund_code: str):
    global data_source
    params = {"code": fund_code}
    FUND_ESTIMATION_BASE_URL = "".join(fund_valuation_url_list)
    fund_base_url = FUND_ESTIMATION_BASE_URL.format(**params)
    r = requests.get(fund_base_url)
    data = r.text
    deal_data = data.replace("jsonpgz(", "").replace(");", "")
    expansion = json.loads(deal_data)
    short_name = expansion.get("name")
    gz = float(expansion.get("gsz"))  # 估值价格
    fund_range = float(expansion.get("gszzl"))  # 估值涨跌幅
    diff_value = round(gz * (float(fund_range) / 100), 3)  # 涨跌值
    gz_time = expansion.get("gztime")  # 估值时间
    data_source.append((short_name, fund_code, gz, diff_value, fund_range, gz_time))


def get_fund_real_time_god(fund_codes: Union[str, List[str]], is_explain: bool = False, **kwargs) -> pd.DataFrame:
    global data_source
    fund_list = [fund_codes] if isinstance(fund_codes, str) else fund_codes
    threads = []
    for item in fund_list:
        threads.append(threading.Thread(target=get_fund_price, args=(item,)))
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    export_data_source = copy.deepcopy(data_source)
    columns = ["基金名称", "基金代码", "净值估算", "涨跌值", "估算涨跌幅", "更新时间"]
    df = pd.DataFrame(export_data_source, columns=columns)
    data_source = []
    return exchange_explain(df, is_explain)



from .capital_flow_monitor import (
    get_stock_history_capital,
    get_stock_real_time_daily_capital,
    get_stock_real_time_sum_capital,
    get_stock_real_time_sector_capital,
)

__all__ = [
    'get_stock_history_capital',
    'get_stock_real_time_daily_capital',
    'get_stock_real_time_sum_capital',
    'get_stock_real_time_sector_capital'
]



import requests
from pathlib import Path
from collections import namedtuple

HERE = Path(__file__).parent
# 数据缓存文件存储目录
DATA_DIR = HERE / '../data'
# 创建数据缓存文件目录
DATA_DIR.mkdir(parents=True, exist_ok=True)
# 搜索词缓存位置
SEARCH_RESULT_CACHE_PATH = str(DATA_DIR / 'search-cache.json')
session = requests.Session()

# 存储证券代码的实体
Quote = namedtuple(
    'Quote',
    [
        'code',
        'name',
        'pinyin',
        'id',
        'jys',
        'classify',
        'market_type',
        'security_typeName',
        'security_type',
        'mkt_num',
        'type_us',
        'quote_id',
        'unified_code',
        'inner_code',
    ],
)


oil_reserves_url = "/oil/reserves"
oil_products_url = "/oil/products"
oil_consumption_url = "/oil/consumption"
oil_refinerythroughput_url = "/oil/refinerythroughput"
oil_refinerycapacity_url = "/oil/refinerycapacity"
oil_crudeoilpricehistory_url = "/oil/crudeoilpricehistory"




oil_reserves_field_dict = {"year": "年份", "num": "储量(十亿桶)", "reserves_pre": "总储量占比",
                      "rp_ratio": "储产比", "short_name": "机构名称", "name": "产品名称",
                      }


oil_products_field_dict = {"year": "年份", "num": "产量(千桶/天)", "product_pre": "总产量占比",
                       "short_name": "机构名称", "name": "产品名称",
                      }


oil_consumption_field_dict = {"year": "年份", "num": "消费量(千桶/天)", "consume_pre": "总产量占比",
                       "short_name": "机构名称", "name": "产品名称",
                      }


oil_refinerythroughput_field_dict = {"year": "年份", "refinery_num": "加工量(千桶/天)", "refinery_pre": "总产量占比",
                       "short_name": "机构名称", "name": "产品名称",
                      }


oil_refinerycapacity_field_dict = {"year": "年份", "capacity_num": "产能(千桶/天)", "capacity_pre": "总产能占比",
                       "short_name": "机构名称", "name": "产品名称",
                      }


oil_crudeoilpricehistory_field_dict = {"year": "年份", "the_day_price": "价格(当年真实价格)", "now_price": "价格(按2022年美国消费者物价指数平减)",
                       "name": "产品名称",
                      }



url = "https://hq.sinajs.cn/?rn=1694440934177&list=globalbd_{bond_name}"

bond_yield_url = "/bond_yield/kline"


headers = {
            "Referer": "https://stock.finance.sina.com.cn/forex/globalbd/gcny10.html"
        }


from .getter import (
    get_stock_kline_data,
    get_stock_real_time_data,
    get_stock_market_real_time_data
)

from .rank_list import get_stock_dragon_tiger_list
from .quarterly_report import get_stock_all_company_quarterly_report, get_stock_all_report_dates, \
    get_stock_company_report_data
from .stock_base_info import get_stock_base_info

__all__ = [
    'get_stock_kline_data',
    'get_stock_real_time_data',
    'get_stock_market_real_time_data',
    'get_stock_dragon_tiger_list',
    'get_stock_all_company_quarterly_report',
    'get_stock_base_info',
    'get_stock_all_report_dates',
    'get_stock_company_report_data'
]


import uuid
import pandas as pd
from retry import retry
from ..utils import to_numeric, requests_obj, validate_request
from .config import EastmoneyFundHeaders
from ...crawl.settings import *
from ..common.explain_change import exchange_explain


@validate_request
@retry(tries=3)
@to_numeric
def get_fund_period_change(token: str, fund_code: str, is_explain: bool = False) -> pd.DataFrame:
    """
    获取基金阶段涨跌幅度
    Parameters
    ----------
    fund_code : str
        6 位基金代码
    Returns
    -------
    DataFrame
        指定基金的阶段涨跌数据
    Examples
    --------
        基金代码     收益率   同类平均  同类排行  同类总数   时间段
    0  161725   -6.28   0.07  1408  1409   近一周
    1  161725   10.85   5.82   178  1382   近一月
    2  161725   25.32   7.10    20  1332   近三月
    3  161725   22.93  10.39    79  1223   近六月
    4  161725  103.76  33.58     7  1118   近一年
    5  161725  166.59  55.42     9   796   近两年
    6  161725  187.50  48.17     2   611   近三年
    7  161725  519.44  61.62     1   389   近五年
    8  161725    6.46   5.03   423  1243  今年以来
    9  161725  477.00                     成立以来
    :param is_explain:
    :param fund_code:
    :param token:
    """
    str_uuid = str(uuid.uuid4()).upper()
    params = (
        ('AppVersion', '6.3.8'),
        ('FCODE', fund_code),
        ('MobileKey', str_uuid),
        ('OSVersion', '14.3'),
        ('deviceid', str_uuid),
        ('passportid', '3061335960830820'),
        ('plat', 'Iphone'),
        ('product', 'EFund'),
        ('version', '6.3.6'),
    )
    json_response = requests_obj.get(''.join(fund_period_change_url), params, headers=EastmoneyFundHeaders).json()
    columns = {
        'syl': '收益率',
        'avg': '同类平均',
        'rank': '同类排行',
        'sc': '同类总数',
        'title': '时间段',
    }
    titles = {
        'Z': '近一周',
        'Y': '近一月',
        '3Y': '近三月',
        '6Y': '近六月',
        '1N': '近一年',
        '2Y': '近两年',
        '3N': '近三年',
        '5N': '近五年',
        'JN': '今年以来',
        'LN': '成立以来',
    }
    # 发行时间
    ESTABDATE = json_response['Expansion']['ESTABDATE']
    df = pd.DataFrame(json_response['Datas'])
    df = df[list(columns.keys())].rename(columns=columns)
    df['时间段'] = titles.values()
    df.insert(0, '基金代码', fund_code)
    return exchange_explain(df, is_explain)



from .fund_list import get_fund_codes
from .get_fund_real_time import get_fund_real_time_god
from .get_fund_history_data import get_fund_history_price
from .get_fund_invest_info import get_fund_invest_position, get_fund_public_dates
from .get_period_change_info import get_fund_period_change
from .get_types_percentage_info import get_fund_types_percentage
from .get_fund_base_info import get_fund_base_info
from .get_fund_industry_info import get_fund_industry_distribution
from .get_fund_rank import (
    get_fund_open_rank,
    get_fund_exchange_rank,
    get_fund_money_rank,
    get_fund_hk_rank
)

__all__ = [
    'get_fund_codes',
    'get_fund_history_price',
    'get_fund_invest_position',
    'get_fund_public_dates',
    'get_fund_period_change',
    'get_fund_types_percentage',
    'get_fund_base_info',
    'get_fund_industry_distribution',
    'get_fund_open_rank',
    'get_fund_exchange_rank',
    'get_fund_money_rank',
    'get_fund_hk_rank',
    'get_fund_real_time_god'
]



from .get_bond_info import (get_bond_base_info_list,
                            get_bond_realtime_quotes,
                            get_bond_base_info,
                            get_bond_public
                            )

# from .get_bond_public_info import (
#     bond_treasure_issue_cninfo,
#     bond_local_government_issue_cninfo,
#     bond_corporate_issue_cninfo,
#     bond_cov_issue_cninfo,
#     bond_cov_stock_issue_cninfo
# )

__all__ = [
    'get_bond_base_info_list',
    'get_bond_realtime_quotes',
    'get_bond_base_info',
    'get_bond_public'
]



user_agent_list = [
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.84 Safari/537.36'
]

quote_id_key_list = ["se", "cid"]

fields1_list = ['f1', 'f2', 'f3', 'f4', 'f5', 'f6', 'f7', 'f8', 'f9', 'f10', 'f11', 'f12', 'f13']

k_url_list = ['htt', 'ps:', '//pu', 'sh', '2hi', 's.eas', 'tmon', 'ey.com', '/api', '/q', 't/st', 'ock/k', 'line/get']

# http://push2.eastmoney.com/api/qt/stock/get

real_time_url_list = ['htt', 'ps:', '//pu', 'sh', '2.ea', 'stmon', 'ey.co', 'm/ap', 'i/q', 't/st', 'oc', 'k/g', 'et']

market_real_time_url_list = ['htt', 'p:/', '/pus', 'h2.e', 'ast', 'mon', 'ey.c', 'om/a', 'pi/q', 't/cl', 'ist/g', 'et']

rank_list_list = ['htt', 'p:/', '/dat', 'acen', 'ter-w', 'eb.ea', 'stmo', 'ney.c', 'om/a', 'pi/d', 'ata/', 'v1/g', 'et']

quarterly_report_url_list = ['htt', 'p:/', '/da', 'tac', 'ente', 'r-w', 'eb.e', 'astm', 'oney.', 'co', 'm/ap', 'i/da', 'ta/g', 'et']

one_quarterly_report_url_list = ['htt', 'ps:/', '/dat', 'acen', 'ter', '-we', 'b.ea', 'stmo', 'ney', '.co', 'm/ap', 'i/da', 'ta/v', '1/g', 'et']

bill_url_list = ['htt', 'p:/', '/pus', 'h2h', 'is.e', 'astm', 'oney', '.co', 'm/a', 'pi/', 'qt/', 'sto', 'ck/', 'fflo', 'w/d', 'aykl', 'ine/g', 'et']

# fund_valuation_url_list = ['ht', 'tps:/', '/fun', 'dmob', 'ap', 'i.eas', 'tmon', 'ey.c', 'om/F', 'undM', 'Ap', 'i/Fu', 'ndV', 'arie', 'tieV', 'alua', 'tio', 'nDet', 'ail.', 'ash', 'x?FC', 'ODE=', '{cod', 'e}&d', 'evic', 'eid=', '{devi', 'cei', 'd}&', 'pla', 't=Iph', 'one&A', 'ppTy', 'pe=I', 'pho', 'ne&p', 'rod', 'uct=', 'EFun', 'd&v', 'ersi', 'on=6', '.2.5']

fund_valuation_url_list = ["https://fundgz.1234567.com.cn/js/{code}.js"]
fund_period_change_url = ['htt', 'ps:/', '/fun', 'dmo', 'bap', 'i.ea', 'stm', 'one', 'y.co', 'm/Fu', 'ndMN', 'ewAp', 'i/Fu', 'ndM', 'NPe', 'rio', 'dIn', 'cre', 'ase']

fund_history_url = ['htt', 'ps:/', '/fu', 'ndmo', 'bap', 'i.ea', 'stmo', 'ney.', 'co', 'm/Fu', 'ndM', 'Ne', 'wAp', 'i/Fu', 'ndM', 'NHi', 'sNe', 'tL', 'ist']

fun_types_percentage_url = ['htt', 'ps:/', '/fun', 'dmo', 'bap', 'i.ea', 'stm', 'oney.', 'co', 'm/Fu', 'ndMN', 'ewA', 'pi/', 'Fun', 'dMN', 'Ass', 'etA', 'llo', 'cati', 'onNew']

fund_basic_url = ['htt', 'ps:/', '/fun', 'dmo', 'bap', 'i.ea', 'stm', 'oney', '.co', 'm/Fu', 'ndMN', 'ewA', 'pi/F' ,'undM', 'NNB', 'asic', 'Info', 'rmat', 'ion']

fund_industry_url = ['htt', 'ps:/', '/fun', 'dmo', 'bap', 'i.ea', 'stm', 'one', 'y.co', 'm/Fu', 'ndM', 'NewA', 'pi/', 'Fun', 'dMNS', 'ecto', 'rAl', 'loc', 'ation']

bond_base_list_url = ['htt', 'p:/', '/dat', 'acen', 'ter', '-we', 'b.ea', 'stmo', 'ney', '.co', 'm/a', 'pi/d', 'ata', '/v', '1/g', 'et']

bond_base_info_url = ['htt', 'p:/', '/dat', 'acen', 'ter-w', 'eb.e', 'astm', 'one', 'y.co', 'm/ap', 'i/da', 'ta/v', '1/g', 'et']

stock_base_info_url = ['htt', 'p:/', '/em', 'we', 'b.se', 'curi', 'ties.', 'eas', 'tmon', 'ey.c', 'om/P', 'C_HS', 'F10/C', 'ompa', 'nySu', 'rve', 'y/Pa', 'geAj', 'ax']

base_url_list = ['htt', 'p:/', '/sha', 'ret', 'op.', 'to', 'p/s', 'har', 'eto', 'papi']
# base_url_list = ["http://127.0.0.1:8000/sharetopapi"]


capital_flow_real_time_url_list = ['htt', 'ps:/', '/pu', 'sh2.e', 'astm', 'oney', '.co', 'm/ap', 'i/q', 't/ul', 'ist.', 'np/g', 'et']

sector_url_list = ['htt', 'ps:/', '/pu', 'sh2', '.eas', 'tmon', 'ey.c', 'om/a', 'pi/q', 't/cl', 'ist', '/g', 'et']



import pandas as pd
import copy
from .config import logistics_index_url, logistics_index_field_dict
from ...crawl.settings import *
from ..utils import to_numeric, requests_obj, parse_obj
from ..common.explain_change import exchange_explain

index_type_list = {
    "1": "电商物流指数"
}


@to_numeric
def get_logistics_data_common(token: str, index_url: str, field_dict: dict, index_type: str = None,
                              start_date: str = None,
                              end_date: str = None, limit: int = None) -> pd.DataFrame:
    """
    :param token:
    :param field_dict: 字段对应关系
    :param car_url:  具体是石油数据url
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    copy_base_url_list = copy.deepcopy(base_url_list)
    copy_base_url_list.append(index_url)
    data = {"index_type": index_type, "start_date": start_date, "end_date": end_date, "limit": limit}
    headers = {"token": token}
    r = requests_obj.get("".join(copy_base_url_list), data=data, headers=headers)
    if isinstance(r, dict):
        return r
    data_json = r.json()
    for item in data_json.get('data', []):
        item['index_type'] = index_type_list.get(item['index_type'], item['index_type'])
    df = parse_obj.parse_god_cup_json(data_json, field_dict)
    return df


def get_logistics_index(token: str, index_type: str = None, start_date: str = None, end_date: str = None,
                        limit: int = 100,
                        is_explain: bool = False) -> pd.DataFrame:
    """
    获取猪周期的历史猪肉价格和猪粮比
    :param is_explain:
    :param index_type:
    :param start_date:
    :param end_date:
    :param token:
    :param limit:
    :return:
    """
    return exchange_explain(
        get_logistics_data_common(token, logistics_index_url, logistics_index_field_dict, index_type,
                                  start_date, end_date, limit), is_explain)



from retry import retry
from ..utils import to_numeric, requests_obj, validate_request
from .config import EastmoneyFundHeaders
from typing import List, Union
from ...crawl.settings import *
import pandas as pd
from ..common.explain_change import exchange_explain


@validate_request
@retry(tries=3)
@to_numeric
def get_fund_types_percentage(
    token: str, fund_code: str, dates: Union[List[str], str, None] = None, is_explain: bool = False
) -> pd.DataFrame:
    """
    获取指定基金不同类型占比信息
    Parameters
    ----------
    fund_code : str
        6 位基金代码
    dates : Union[List[str], str, None]
        可选值类型示例如下(后面有获取 dates 的例子)

        - ``None`` : 最新公开日期
        - ``'2020-01-01'`` : 一个公开持仓日期
        - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
    Returns
    -------
    DataFrame
        指定基金的在不同日期的不同类型持仓占比信息
    Examples
    --------
        基金代码   股票比重 债券比重  现金比重         总规模(亿元) 其他比重
    0  005827   94.4   --  6.06  880.1570625231    0
    0  005827  94.09   --  7.63   677.007455712    0
    :param dates:
    :param fund_code:
    :param token:
    """
    columns = {'GP': '股票比重', 'ZQ': '债券比重', 'HB': '现金比重', 'JZC': '总规模(亿元)', 'QT': '其他比重'}
    df = pd.DataFrame(columns=columns.values())
    if not isinstance(dates, List):
        dates = [dates]
    elif dates is None:
        dates = [None]
    for date in dates:
        params = [
            ('FCODE', fund_code),
            ('OSVersion', '14.3'),
            ('appVersion', '6.3.8'),
            ('deviceid', '3EA024C2-7F21-408B-95E4-383D38160FB3'),
            ('plat', 'Iphone'),
            ('product', 'EFund'),
            ('serverVersion', '6.3.6'),
            ('version', '6.3.8'),
        ]
        if date is not None:
            params.append(('DATE', date))
        params = tuple(params)
        url = ''.join(fun_types_percentage_url)
        json_response = requests_obj.get(url, params, headers=EastmoneyFundHeaders).json()
        if len(json_response['Datas']) == 0:
            continue
        _df = pd.DataFrame(json_response['Datas'])[columns.keys()]
        _df = _df.rename(columns=columns)
        df = pd.concat([df, _df], axis=0, ignore_index=True)
    df.insert(0, '基金代码', fund_code)
    return exchange_explain(df, is_explain)





country_base_url = "/country/data_base"

country_field_dict = {"short_name": "国家简称", "full_name": "国家全称", "en_short_name": "英文简称",
                      "en_full_name": "英文全称", "land_area": "土地面积(万平方公里)", "continent": "所属洲",
                      "country_type": "机构类型", "abbreviation": "国家简称", "capital_cn": "首都(中文)",
                      "capital_en": "首都(英文)", "org_name": "所属组织"}



import pandas as pd
from typing import Dict, List, Union
from ..common.common_base import CommonFunc
from ..common.explain_change import exchange_explain
from ..utils import validate_request

common_func_obj = CommonFunc()


def get_stock_base_info(stock_codes: Union[str, List[str]], is_explain: bool = False
) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
    """
    获取股票的基本信息
    :param is_explain:
    :param stock_codes: 一个或者多个股票代码
    :return: 股票的基本信息
    """
    base_func_name = get_stock_base_info.__name__
    return exchange_explain(common_func_obj.get_common_func(stock_codes, base_func_name), is_explain)



import os
from ftplib import FTP  # 引入ftp模块


class MyFtp:
    ftp = FTP()

    def __init__(self, host, port=2121):
        self.ftp.connect(host, port)

    def login(self, username, pwd):
        self.ftp.set_debuglevel(2)  # 打开调试级别2，显示详细信息        
        self.ftp.login(username, pwd)
        print(self.ftp.welcome)

    def downloadFile(self, localpath, remotepath, filename):
        os.chdir(localpath)  # 切换工作路径到下载目录
        self.ftp.cwd(remotepath)  # 要登录的ftp目录
        self.ftp.nlst()  # 获取目录下的文件
        file_handle = open(filename, "wb").write  # 以写模式在本地打开文件
        self.ftp.retrbinary('RETR %s' % os.path.basename(filename), file_handle, blocksize=1024)  # 下载ftp文件
        # ftp.delete（filename）  # 删除ftp服务器上的文件

    def close(self):
        self.ftp.set_debuglevel(0)  # 关闭调试        
        self.ftp.quit()


if __name__ == '__main__':
    ftp = MyFtp('183.6.56.70')
    ftp.login('ftp-ths-read', 'i!9|j3pB')
    ftp.downloadFile('/', '/2023-12-25/insert/', '1703471104207.json')
    ftp.close()



import pandas as pd
import copy
from .config import oil_reserves_url, oil_reserves_field_dict, oil_products_field_dict, oil_products_url, \
    oil_consumption_url, oil_consumption_field_dict, oil_refinerythroughput_field_dict, oil_refinerythroughput_url, \
    oil_refinerycapacity_url, oil_refinerycapacity_field_dict, oil_crudeoilpricehistory_field_dict, \
    oil_crudeoilpricehistory_url
from ...crawl.settings import *
from ..utils import to_numeric, requests_obj, parse_obj
from ..common.explain_change import exchange_explain


@to_numeric
def get_oil_data_common(token: str, oil_url: str, field_dict: dict, data_type: str = None,
                        limit: int = None) -> pd.DataFrame:
    """
    :param token:
    :param field_dict: 字段对应关系
    :param oil_url:  具体是石油数据url
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    copy_base_url_list = copy.deepcopy(base_url_list)
    copy_base_url_list.append(oil_url)
    data = {}
    headers = {"token": token}
    if data_type:
        data["data_type"] = data_type
    if limit:
        data['limit'] = limit
    r = requests_obj.get("".join(copy_base_url_list), data=data, headers=headers)
    if isinstance(r, dict):
        return r
    data_json = r.json()
    df = parse_obj.parse_god_cup_json(data_json, field_dict)
    return df


def get_oil_reserves(token: str, data_type: str = '1', limit: int = 100, is_explain: bool = False) -> pd.DataFrame:
    """
    获取机构的石油储量
    :param token:
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    return exchange_explain(get_oil_data_common(token, oil_reserves_url, oil_reserves_field_dict, data_type, limit), is_explain)


def get_oil_products(token: str, data_type: str = '1', limit: int = 100, is_explain: bool = False) -> pd.DataFrame:
    """
    获取机构的石油产量
    :param token:
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    return exchange_explain(get_oil_data_common(token, oil_products_url, oil_products_field_dict, data_type, limit), is_explain)


def get_oil_consumption(token: str, data_type: str = '1', limit: int = 100, is_explain: bool = False) -> pd.DataFrame:
    """
    获取机构的消费量
    :param token:
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    return exchange_explain(get_oil_data_common(token, oil_consumption_url, oil_consumption_field_dict, data_type, limit), is_explain)


def get_oil_refinerythroughput(token: str, data_type: str = '1', limit: int = 100, is_explain: bool = False) -> pd.DataFrame:
    """
    获取石油的加工量
    :param token:
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    return exchange_explain(get_oil_data_common(token, oil_refinerythroughput_url, oil_refinerythroughput_field_dict, data_type, limit), is_explain)


def get_oil_refinerycapacity(token: str, data_type: str = '1', limit: int = 100, is_explain: bool = False) -> pd.DataFrame:
    """
    获取石油的产能
    :param token:
    :param data_type: 1为国家数据 2为机构数据 4为大洲数据
    :param limit:
    :return:
    """
    return exchange_explain(get_oil_data_common(token, oil_refinerycapacity_url, oil_refinerycapacity_field_dict, data_type, limit), is_explain)


def get_oil_crudeoilpricehistory(token: str, limit: int = 100, is_explain: bool = False) -> pd.DataFrame:
    """
    获取石油的历史价格
    :param token:
    :param limit: 数据限制
    :return:
    """
    return exchange_explain(get_oil_data_common(token, oil_crudeoilpricehistory_url, oil_crudeoilpricehistory_field_dict, limit=limit), is_explain)



from .fund_list import get_fund_codes
from .get_fund_rank import get_fund_exchange_rank


class FundServices:
    def __init__(self, token, *args, **kwargs):
        self.token = token

    def fund_to_zq(self, **kwargs):
        is_explain = kwargs.get("is_explain", False)
        params = {'is_explain': is_explain, "ft": "zq", "token": self.token}
        return get_fund_codes(**params)

    def fund_to_etf(self, **kwargs):
        is_explain = kwargs.get("is_explain", False)
        params = {'is_explain': is_explain, "token": self.token}
        fund_name_field = "基金简称" if is_explain else "fund_name"
        df = get_fund_exchange_rank(**params)
        new_df = df[df[fund_name_field].str.contains('国债', case=False)]
        return new_df



import datetime
import copy
import pandas as pd
from ..utils import requests_obj
from .config import pig_warning_url, pig_fcr_url
from ..common.explain_change import exchange_explain_new
from ...crawl.settings import *
from sharetop.core.stock import get_stock_kline_data


class PigServices:
    def __init__(self, token, *args, **kwargs):
        self.token = token

    def pig_to_warning_kline_data(self, **kwargs):
        is_explain = kwargs.pop("is_explain", False)
        return exchange_explain_new(self._common(pig_warning_url, kwargs), is_explain)

    def pig_to_fcr_kline_data(self, **kwargs):
        is_explain = kwargs.pop("is_explain", False)
        return exchange_explain_new(self._common(pig_fcr_url, kwargs), is_explain)

    def _common(self, sub_url, item):
        pig_base_url_list = copy.deepcopy(base_url_list)
        pig_base_url_list.append(sub_url)
        headers = {"token": self.token}
        r = requests_obj.get("".join(pig_base_url_list), data=item, headers=headers)
        if isinstance(r, dict):
            return r
        data_json = r.json()
        df = pd.DataFrame(data_json['data'])
        return df

    def pig_to_index(self, **kwargs):
        is_explain = kwargs.get("is_explain", False)
        params = {'is_explain': is_explain, "stock_codes": "BK0882", "token": self.token}
        return get_stock_kline_data(**params)



import requests
import datetime
import re
import pandas as pd
from ..utils import requests_obj, parse_obj, validate_request, to_numeric
from .config import fund_money_cloumns
from ..common.explain_change import exchange_explain


def fund_export_df(text_data):
    json_data = parse_obj.parse_fund_json(text_data)
    temp_df = pd.DataFrame(json_data)
    temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    return temp_df


@to_numeric
@validate_request
def get_fund_open_rank(token: str, symbol: str = "全部", is_explain: bool = False) -> pd.DataFrame:
    """
    东方财富网-数据中心-开放基金排行
    https://fund.eastmoney.com/data/fundranking.html
    :param token:
    :param is_explain:
    :param symbol: choice of {"全部", "股票型", "混合型", "债券型", "指数型", "QDII", "LOF", "FOF"}
    :type symbol: str
    :return: 开放基金排行
    :rtype: pandas.DataFrame
    """
    current_date = datetime.datetime.now().date().isoformat()
    last_date = str(int(current_date[:4]) - 1) + current_date[4:]
    url = "http://fund.eastmoney.com/data/rankhandler.aspx"
    type_map = {
        "全部": ["all", "zzf"],
        "股票型": ["gp", "6yzf"],
        "混合型": ["hh", "6yzf"],
        "债券型": ["zq", "6yzf"],
        "指数型": ["zs", "6yzf"],
        "QDII": ["qdii", "6yzf"],
        "LOF": ["lof", "6yzf"],
        "FOF": ["fof", "6yzf"],
    }
    params = {
        "op": "ph",
        "dt": "kf",
        "ft": type_map[symbol][0],
        "rs": "",
        "gs": "0",
        "sc": type_map[symbol][1],
        "st": "desc",
        "sd": last_date,
        "ed": current_date,
        "qdii": "",
        "tabSubtype": ",,,,,",
        "pi": "1",
        "pn": "20000",
        "dx": "1",
        "v": "0.1591891419018292",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "http://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests_obj.get(url, params, headers=headers)
    text_data = r.text
    temp_df = fund_export_df(text_data)
    temp_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "_",
        "日期",
        "单位净值",
        "累计净值",
        "日增长率",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "_",
        "_",
        "自定义",
        "_",
        "手续费",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "日期",
            "单位净值",
            "累计净值",
            "日增长率",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "手续费",
        ]
    ]
    return exchange_explain(temp_df, is_explain)


@to_numeric
@validate_request
def get_fund_exchange_rank(token: str, is_explain: bool = False) -> pd.DataFrame:
    """
    东方财富网-数据中心-场内交易基金排行
    https://fund.eastmoney.com/data/fbsfundranking.html
    :return: 场内交易基金数据
    :rtype: pandas.DataFrame
    """
    url = "http://fund.eastmoney.com/data/rankhandler.aspx"
    params = {
        "op": "ph",
        "dt": "fb",
        "ft": "ct",
        "rs": "",
        "gs": "0",
        "sc": "1nzf",
        "st": "desc",
        "pi": "1",
        "pn": "10000",
        "v": "0.1591891419018292",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "http://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests_obj.get(url, params, headers=headers)
    text_data = r.text
    temp_df = fund_export_df(text_data)
    temp_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "_",
        "日期",
        "单位净值",
        "累计净值",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "成立日期",
        "_",
        "_",
        "_",
        "_",
        "_",
        "类型",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "类型",
            "日期",
            "单位净值",
            "累计净值",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "成立日期",
        ]
    ]
    # temp_df['日期'] = pd.to_datetime(temp_df['日期']).dt.date
    # temp_df['成立日期'] = pd.to_datetime(temp_df['成立日期']).dt.date
    # temp_df['单位净值'] = pd.to_numeric(temp_df['单位净值'], errors="coerce")
    # temp_df['累计净值'] = pd.to_numeric(temp_df['累计净值'], errors="coerce")
    # temp_df['近1周'] = pd.to_numeric(temp_df['近1周'], errors="coerce")
    # temp_df['近1月'] = pd.to_numeric(temp_df['近1月'], errors="coerce")
    # temp_df['近3月'] = pd.to_numeric(temp_df['近3月'], errors="coerce")
    # temp_df['近6月'] = pd.to_numeric(temp_df['近6月'], errors="coerce")
    # temp_df['近1年'] = pd.to_numeric(temp_df['近1年'], errors="coerce")
    # temp_df['近2年'] = pd.to_numeric(temp_df['近2年'], errors="coerce")
    # temp_df['近3年'] = pd.to_numeric(temp_df['近3年'], errors="coerce")
    # temp_df['今年来'] = pd.to_numeric(temp_df['今年来'], errors="coerce")
    # temp_df['成立来'] = pd.to_numeric(temp_df['成立来'], errors="coerce")
    return exchange_explain(temp_df, is_explain)


@to_numeric
@validate_request
def get_fund_money_rank(token: str, is_explain: bool = False) -> pd.DataFrame:
    """
    东方财富网-数据中心-货币型基金排行
    https://fund.eastmoney.com/data/hbxfundranking.html
    :return: 货币型基金排行
    :rtype: pandas.DataFrame
    """
    url = "http://api.fund.eastmoney.com/FundRank/GetHbRankList"
    params = {
        "intCompany": "0",
        "MinsgType": "",
        "IsSale": "1",
        "strSortCol": "SYL_1N",
        "orderType": "desc",
        "pageIndex": "1",
        "pageSize": "10000",
        "_": "1603867224251",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "http://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests_obj.get(url, params, headers=headers)
    json_data = r.json()
    temp_data_json = []
    for _ in json_data["Data"]:
        temp_dict = {}
        for k, v in _.items():
            if k in fund_money_cloumns:
                temp_dict[k] = v
        temp_data_json.append(temp_dict)
    temp_df = pd.DataFrame(temp_data_json)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.rename(columns=fund_money_cloumns, inplace=True)
    # temp_df['日期'] = pd.to_datetime(temp_df['日期']).dt.date
    # temp_df['万份收益'] = pd.to_numeric(temp_df['万份收益'], errors="coerce")
    # temp_df['年化收益率7日'] = pd.to_numeric(temp_df['年化收益率7日'], errors="coerce")
    # temp_df['年化收益率14日'] = pd.to_numeric(temp_df['年化收益率14日'], errors="coerce")
    # temp_df['年化收益率28日'] = pd.to_numeric(temp_df['年化收益率28日'], errors="coerce")
    # temp_df['近1月'] = pd.to_numeric(temp_df['近1月'], errors="coerce")
    # temp_df['近3月'] = pd.to_numeric(temp_df['近3月'], errors="coerce")
    # temp_df['近6月'] = pd.to_numeric(temp_df['近6月'], errors="coerce")
    # temp_df['近1年'] = pd.to_numeric(temp_df['近1年'], errors="coerce")
    # temp_df['近2年'] = pd.to_numeric(temp_df['近2年'], errors="coerce")
    # temp_df['近3年'] = pd.to_numeric(temp_df['近3年'], errors="coerce")
    # temp_df['近5年'] = pd.to_numeric(temp_df['近5年'], errors="coerce")
    # temp_df['今年来'] = pd.to_numeric(temp_df['今年来'], errors="coerce")
    # temp_df['成立来'] = pd.to_numeric(temp_df['成立来'], errors="coerce")
    return exchange_explain(temp_df, is_explain)


@to_numeric
@validate_request
def get_fund_hk_rank(token: str, is_explain: bool = False) -> pd.DataFrame:
    """
    东方财富网-数据中心-香港基金排行
    https://overseas.1234567.com.cn/FundList
    :return: 香港基金排行
    :rtype: pandas.DataFrame
    """
    format_date = datetime.datetime.now().date().isoformat()
    url = "https://overseas.1234567.com.cn/overseasapi/OpenApiHander.ashx"
    params = {
        'api': 'HKFDApi',
        'm': 'MethodFundList',
        'action': '1',
        'pageindex': '0',
        'pagesize': '5000',
        'dy': '1',
        'date1': format_date,
        'date2': format_date,
        'sortfield': 'Y',
        'sorttype': '-1',
        'isbuy': '0',
        '_': '1610790553848',
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "http://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests_obj.get(url, params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "_",
        "香港基金代码",
        "基金代码",
        "_",
        "基金简称",
        "可购买",
        "日期",
        "单位净值",
        "日增长率",
        "_",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "币种",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "币种",
            "日期",
            "单位净值",
            "日增长率",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "可购买",
            "香港基金代码",
        ]
    ]
    # temp_df['日期'] = pd.to_datetime(temp_df['日期']).dt.date
    temp_df['单位净值'] = pd.to_numeric(temp_df['单位净值'], errors="coerce")
    temp_df['日增长率'] = pd.to_numeric(temp_df['日增长率'], errors="coerce")
    # temp_df['近1周'] = pd.to_numeric(temp_df['近1周'], errors="coerce")
    # temp_df['近1月'] = pd.to_numeric(temp_df['近1月'], errors="coerce")
    # temp_df['近3月'] = pd.to_numeric(temp_df['近3月'], errors="coerce")
    # temp_df['近6月'] = pd.to_numeric(temp_df['近6月'], errors="coerce")
    # temp_df['近1年'] = pd.to_numeric(temp_df['近1年'], errors="coerce")
    # temp_df['近2年'] = pd.to_numeric(temp_df['近2年'], errors="coerce")
    # temp_df['近3年'] = pd.to_numeric(temp_df['近3年'], errors="coerce")
    # temp_df['今年来'] = pd.to_numeric(temp_df['今年来'], errors="coerce")
    # temp_df['成立来'] = pd.to_numeric(temp_df['成立来'], errors="coerce")
    # temp_df['成立来'] = pd.to_numeric(temp_df['成立来'], errors="coerce")
    temp_df['可购买'] = temp_df['可购买'].map(lambda x: "可购买" if x == "1" else "不可购买")
    return exchange_explain(temp_df, is_explain)



import uuid
import pandas as pd
from typing import List, Union
from ..utils import to_numeric, requests_obj, validate_request
from ...crawl.settings import *
from .config import EastmoneyFundHeaders
from ..common.explain_change import exchange_explain


@validate_request
@to_numeric
def get_fund_industry_distribution(
    token: str, fund_code: str, dates: Union[str, List[str]] = None, is_explain: bool = False
) -> pd.DataFrame:
    """
    获取指定基金行业分布信息
    Parameters
    ----------
    fund_code : str
        6 位基金代码
    dates : Union[str, List[str]], optional
        日期
        可选示例如下
        - ``None`` : 最新公开日期
        - ``'2020-01-01'`` : 一个公开持仓日期
        - ``['2020-12-31' ,'2019-12-31']`` : 多个公开持仓日期
    Returns
    -------
    DataFrame
        指定基金行业持仓信息
    Examples
    --------
    0   161725               制造业  93.07  2021-06-30  6492580.019556
    1   161725               金融业   0.01  2021-06-30      485.060688
    2   161725          农、林、牧、渔业      0  2021-06-30        0.585078
    3   161725  电力、热力、燃气及水生产和供应业      0  2021-06-30        1.302039
    4   161725               建筑业      0  2021-06-30        2.537137
    5   161725            批发和零售业      0  2021-06-30        5.888394
    6   161725   信息传输、软件和信息技术服务业      0  2021-06-30      157.037536
    7   161725     水利、环境和公共设施管理业      0  2021-06-30        4.443833
    8   161725                教育      0  2021-06-30        1.626203
    9   161725        科学研究和技术服务业      0  2021-06-30        48.30805
    10  161725               采矿业     --  2021-06-30              --
    11  161725       交通运输、仓储和邮政业     --  2021-06-30              --
    12  161725          租赁和商务服务业     --  2021-06-30              --
    13  161725            住宿和餐饮业     --  2021-06-30              --
    14  161725              房地产业     --  2021-06-30              --
    15  161725     居民服务、修理和其他服务业     --  2021-06-30              --
    16  161725           卫生和社会工作     --  2021-06-30              --
    17  161725         文化、体育和娱乐业     --  2021-06-30              --
    18  161725                综合     --  2021-06-30              --
    19  161725                合计  93.08  2021-06-30  6493286.808514
    :param is_explain:
    :param dates:
    :param fund_code:
    :param token:
    """
    columns = {'HYMC': '行业名称', 'ZJZBL': '持仓比例', 'FSRQ': '公布日期', 'SZ': '市值'}
    str_uuid = str(uuid.uuid4()).upper()
    df = pd.DataFrame(columns=columns.values())
    if isinstance(dates, str):
        dates = [dates]
    elif dates is None:
        dates = [None]
    for date in dates:
        params = [
            ('FCODE', fund_code),
            ('OSVersion', '14.4'),
            ('appVersion', '6.3.8'),
            ('deviceid', str_uuid),
            ('plat', 'Iphone'),
            ('product', 'EFund'),
            ('serverVersion', '6.3.6'),
            ('version', '6.3.8'),
        ]
        if date is not None:
            params.append(('DATE', date))
        url = "".join(fund_industry_url)
        response = requests_obj.get(url, params, headers=EastmoneyFundHeaders)
        datas = response.json()['Datas']
        _df = pd.DataFrame(datas)
        _df = _df.rename(columns=columns)
        df = pd.concat([df, _df], axis=0, ignore_index=True)
    df.insert(0, '基金代码', fund_code)
    df = df.drop_duplicates()
    return exchange_explain(df, is_explain)



EastmoneyFundHeaders = {
    'User-Agent': 'EMProjJijin/6.2.8 (iPhone; iOS 13.6; Scale/2.00)',
    'GTOKEN': '98B423068C1F4DEF9842F82ADF08C5db',
    'clientInfo': 'ttjj-iPhone10,1-iOS-iOS13.6',
    'Content-Type': 'application/x-www-form-urlencoded',
    'Host': 'fundmobapi.eastmoney.com',
    'Referer': 'https://mpservice.com/516939c37bdb4ba2b1138c50cf69a2e1/release/pages/FundHistoryNetWorth',
}


fund_money_cloumns = {
    "SHORTNAME": "基金简称",
    "FCODE": "基金代码",
    "FSRQ": "日期",
    "DWJZ": "万份收益",
    "LJJZ": "年化收益率7日",
    "FTYI": "年化收益率14日",
    "TEYI": "年化收益率28日",
    "SYL_Y": "近1月",
    "SYL_3Y": "近3月",
    "SYL_6Y": "近6月",
    "SYL_1N": "近1年",
    "SYL_2N": "近2年",
    "SYL_3N": "近3年",
    "SYL_5N": "近5年",
    "SYL_JN": "今年来",
    "SYL_LN": "成立来",
    "index": '序号'
}




import traceback
import gevent
from gevent.pool import Pool
import pandas as pd
from jsonpath import jsonpath
from .config import (EM_KLINE_FIELDS,
                     MagicConfig,
                     EASTMONEY_REQUEST_HEADERS,
                     EM_REAL_TIME_FIELDS_PARAMS,
                     EM_REAL_TIME_FIELDS,
                     EASTMONEY_QUOTE_FIELDS,
                     QUARTERLY_DICT,
                     EASTMONEY_HISTORY_BILL_FIELDS
                     )
from ..utils import get_quote_id, to_numeric, requests_obj
from typing import Dict, List, Union
from ...application import BaseApplication
from ...crawl.settings import *
from .config import EASTMONEY_BASE_INFO_FIELDS, BASE_INFO_CACHE


@to_numeric
def get_real_time_bill_data_one(code: str) -> pd.DataFrame:
    """
    获取单只股票最新交易日的日内分钟级单子流入流出数据
    Parameters
    ----------
    code : str
        股票、债券代码

    Returns
    -------
    DataFrame
        单只股票、债券最新交易日的日内分钟级单子流入流出数据
    """
    quote_id = get_quote_id(code)
    params = (
        ('lmt', '0'),
        ('klt', '1'),
        ('secid', quote_id),
        ('fields1', 'f1,f2,f3,f7'),
        ('fields2', 'f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63'),
    )
    url = 'http://push2.eastmoney.com/api/qt/stock/fflow/kline/get'
    json_response = requests_obj.get(url, params).json()
    columns = ['时间', '主力净流入', '小单净流入', '中单净流入', '大单净流入', '超大单净流入']
    name = jsonpath(json_response, '$..name')[0]
    code = quote_id.split('.')[-1]
    klines: List[str] = jsonpath(json_response, '$..klines[:]')
    if not klines:
        columns.insert(0, '股票代码')
        columns.insert(0, '股票名称')
        return pd.DataFrame(columns=columns)
    rows = [kline.split(',') for kline in klines]
    df = pd.DataFrame(rows, columns=columns)
    df.insert(0, '股票代码', code)
    df.insert(0, '股票名称', name)
    return df


@to_numeric
def get_history_bill(code: str) -> pd.DataFrame:
    """
    获取单支股票、债券的历史单子流入流出数据

    Parameters
    ----------
    code : str
        股票、债券代码

    Returns
    -------
    DataFrame
        沪深市场单只股票、债券历史单子流入流出数据

    """
    fields = list(EASTMONEY_HISTORY_BILL_FIELDS.keys())
    columns = list(EASTMONEY_HISTORY_BILL_FIELDS.values())
    fields2 = ",".join(fields)
    quote_id = get_quote_id(code)
    params = (
        ('lmt', '100000'),
        ('klt', '101'),
        ('secid', quote_id),
        ('fields1', 'f1,f2,f3,f7'),
        ('fields2', fields2),
    )
    json_response = requests_obj.get("".join(bill_url_list), params).json()
    application_obj = BaseApplication(json_response)
    return application_obj.deal_bill(columns, quote_id)


@to_numeric
def get_company_report_data_one(code: str, report_class: str = None, **kwargs) -> pd.DataFrame:
    columns = QUARTERLY_DICT
    params = {
        'sortColumns': 'REPORTDATE',
        'sortTypes': '-1',
        'pageSize': '200',
        'columns': 'ALL',
        'filter': f'(SECURITY_CODE={code})(DATEMMDD="{report_class}")' if report_class else f'(SECURITY_CODE={code})',
        'reportName': 'RPT_LICO_FN_CPD',
    }
    json_response = requests_obj.get(
        ''.join(one_quarterly_report_url_list), params, user_agent=False
    ).json()
    application_obj = BaseApplication(json_response)
    return application_obj.deal_quarterly_report(columns)


def get_company_report(stock_codes: Union[str, List[str]],
                       report_class: str = None, **kwargs):
    if isinstance(stock_codes, str):
        return get_company_report_data_one(stock_codes, report_class)
    elif hasattr(stock_codes, '__iter__'):
        codes = list(stock_codes)
        return get_quote_multi(
            codes, report_class=report_class, **kwargs
        )
    raise TypeError('代码数据类型输入不正确！')


@to_numeric
def get_market_realtime_by_fs(fs: str, **kwargs) -> pd.DataFrame:
    """
    获取沪深市场最新行情总体情况

    Returns
    -------
    DataFrame
        沪深市场最新行情信息（涨跌幅、换手率等信息）

    """
    columns = {**EASTMONEY_QUOTE_FIELDS, **kwargs.get(MagicConfig.EXTRA_FIELDS, {})}
    fields = ",".join(columns.keys())
    params = (
        ('pn', '1'),
        ('pz', '1000000'),
        ('po', '1'),
        ('np', '1'),
        ('fltt', '2'),
        ('invt', '2'),
        ('fid', 'f3'),
        ('fs', fs),
        ('fields', fields),
    )
    url = ''.join(market_real_time_url_list)
    json_response = requests_obj.get(
        url, params, user_agent=False
    ).json()
    application_obj = BaseApplication(json_response)
    return application_obj.deal_market_realtime(columns)


def quote_id_process(params, quote_id):
    quote_id_key = ''.join(quote_id_key_list)
    params.update(
        {quote_id_key: quote_id}
    )
    return params


def get_real_time(
        stock_codes: Union[str, List[str]],
        **kwargs
):
    if isinstance(stock_codes, str):
        return get_real_time_data_one(stock_codes)
    elif hasattr(stock_codes, '__iter__'):
        codes = list(stock_codes)
        return get_quote_multi(
            codes, **kwargs
        )
    raise TypeError('代码数据类型输入不正确！')


@to_numeric
def get_real_time_data_one(
        code: str,
        **kwargs
) -> pd.DataFrame:
    """
        获取单只股票、债券 实时的基本数据
    """
    columns = list(EM_REAL_TIME_FIELDS.values())
    quote_id = get_quote_id(code)
    params = {
        "fields": EM_REAL_TIME_FIELDS_PARAMS
    }
    params = quote_id_process(params, quote_id)
    url = ''.join(real_time_url_list)
    json_response = requests_obj.get(
        url, params, user_agent=False
    ).json()
    application_obj = BaseApplication(json_response)
    return application_obj.deal_real_time_data(columns, EM_REAL_TIME_FIELDS)


@to_numeric
def get_history_data_one(
        code: str,
        beg: str = '19000101',
        end: str = '20500101',
        klt: int = 101,
        fqt: int = 1,
        **kwargs,
) -> pd.DataFrame:
    """
    获取单只股票、债券 K 线数据
    """
    fields = list(EM_KLINE_FIELDS.keys())
    columns = list(EM_KLINE_FIELDS.values())
    fields2 = ",".join(fields)
    if kwargs.get(MagicConfig.QUOTE_ID_MODE) or "." in code:
        quote_id = code
    else:
        quote_id = get_quote_id(code)
    params = {
        'fields1': ','.join(fields1_list),
        'fields2': fields2,
        'beg': beg,
        'end': end,
        'rtntype': '6',
        'klt': f'{klt}',
        'fqt': f'{fqt}',
    }
    params = quote_id_process(params, quote_id)
    url = ''.join(k_url_list)
    json_response = requests_obj.get(
        url, params, user_agent=True
    ).json()
    application_obj = BaseApplication(json_response)
    return application_obj.deal_k_data(columns, quote_id)


def get_history(
        codes: Union[str, List[str]],
        beg: str = '19000101',
        end: str = '20500101',
        klt: int = 101,
        fqt: int = 1,
        **kwargs,
) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
    """
    获取股票、ETF、债券的 K 线数据

    Parameters
    ----------
    codes : Union[str,List[str]]
        股票、债券代码 或者 代码构成的列表
    beg : str, optional
        开始日期，默认为 ``'19000101'`` ，表示 1900年1月1日
    end : str, optional
        结束日期，默认为 ``'20500101'`` ，表示 2050年1月1日
    klt : int, optional
        行情之间的时间间隔，默认为 ``101`` ，可选示例如下

        - ``1`` : 分钟
        - ``5`` : 5 分钟
        - ``15`` : 15 分钟
        - ``30`` : 30 分钟
        - ``60`` : 60 分钟
        - ``101`` : 日
        - ``102`` : 周
        - ``103`` : 月

    fqt : int, optional
        复权方式，默认为 ``1`` ，可选示例如下

        - ``0`` : 不复权
        - ``1`` : 前复权
        - ``2`` : 后复权

    Returns
    -------
    Union[DataFrame, Dict[str, DataFrame]]
        股票、债券的 K 线数据

        - ``DataFrame`` : 当 ``codes`` 是 ``str`` 时
        - ``Dict[str, DataFrame]`` : 当 ``codes`` 是 ``List[str]`` 时

    """

    if isinstance(codes, str):
        return get_history_data_one(
            codes, beg=beg, end=end, klt=klt, fqt=fqt, **kwargs
        )

    elif hasattr(codes, '__iter__'):
        codes = list(codes)
        return get_quote_multi(
            codes, beg=beg, end=end, klt=klt, fqt=fqt, **kwargs
        )
    raise TypeError('代码数据类型输入不正确！')


def get_quote_multi(
        codes: List[str],
        beg: str = '19000101',
        end: str = '20500101',
        klt: int = 101,
        fqt: int = 1,
        report_class: str = None,
        tries: int = 3,
        **kwargs,
) -> Dict[str, pd.DataFrame]:
    """
    获取多只股票、债券历史行情信息 或者实时行情
    """
    s = traceback.extract_stack()
    base_func_name = s[-2][2]
    total = len(codes)
    pool = Pool(total)
    func = globals()[f"{base_func_name}_data_one"]
    coroutine_list = [pool.spawn(func, x, beg=beg, end=end, klt=klt, fqt=fqt, report_class=report_class, **kwargs) for x
                      in codes]
    gevent.joinall(coroutine_list)
    df_value = [_.value for _ in coroutine_list]
    dfs = dict(zip(codes, df_value))
    return dfs


@to_numeric
def get_base_info(quote_id: str) -> pd.Series:
    fields = ",".join(EASTMONEY_BASE_INFO_FIELDS.keys())
    params = (
        ('ut', 'fa5fd1943c7b386f172d6893dbfba10b'),
        ('invt', '2'),
        ('fltt', '2'),
        ('fields', fields),
        ('secid', quote_id),
    )
    url = 'http://push2.eastmoney.com/api/qt/stock/get'
    json_response = requests_obj.get(url, params, headers=EASTMONEY_REQUEST_HEADERS).json()
    items = json_response['data']
    if not items:
        return pd.Series(index=EASTMONEY_BASE_INFO_FIELDS.values(), dtype='object')
    s = pd.Series(items, dtype='object').rename(index=EASTMONEY_BASE_INFO_FIELDS)
    return s


@to_numeric
def get_deal_detail(quote_id: str, max_count: int = 1000000) -> pd.DataFrame:
    """
    获取股票、期货、债券的最新交易日成交情况
    Parameters
    ----------
    quote_id : str
        包含市场编号的股票、期货、债券代码
    max_count : int, optional
        最大数据条数, 默认为 ``1000000``
    Returns
    -------
    DataFrame
        股票、期货、债券的最新交易日成交情况
    Notes
    ------
    返回的数据表头: ``['名称', '代码', '时间', '昨收', '成交价', '成交量', '单数']``
    """
    base_info = BASE_INFO_CACHE.get(quote_id, get_base_info(quote_id))
    BASE_INFO_CACHE[quote_id] = base_info
    columns = ['名称', '代码', '时间', '昨收', '成交价', '成交量', '单数']
    if str(base_info['代码']).lower() == 'nan':
        return pd.DataFrame(columns=columns)
    code = base_info['代码']
    name = base_info['名称']
    params = (
        ('secid', quote_id),
        ('fields1', 'f1,f2,f3,f4,f5'),
        ('fields2', 'f51,f52,f53,f54,f55'),
        ('pos', f'-{int(max_count)}'),
    )
    url = 'https://push2.eastmoney.com/api/qt/stock/details/get'
    json_response = requests_obj.get(url, params).json()
    # print("json_response==================:", json_response)
    lines: List[str] = json_response['data']['details']
    rows = [line.split(',')[:4] for line in lines]
    df = pd.DataFrame(columns=columns, index=range(len(rows)))
    df.loc[:, '代码'] = code
    df.loc[:, '名称'] = name
    detail_df = pd.DataFrame(rows, columns=['时间', '成交价', '成交量', '单数'])
    detail_df.insert(1, '昨收', json_response['data']['prePrice'])
    df.loc[:, detail_df.columns] = detail_df.values
    return df



import glob
import importlib
import os

__all__ = ["BasicTop"]

import pandas as pd


def _camel_to_snake(camel_case_str):
    """驼峰命名 转换 下划线命名"""
    snake_case_str = ""
    for index in range(len(camel_case_str)):
        char = camel_case_str[index]
        if char.isupper() and index == 0:
            snake_case_str += char.lower()
        elif char.isupper() and index != 0:
            snake_case_str += "_" + char.lower()
        else:
            snake_case_str += char
    return snake_case_str


def _get_all_services_map():
    """获取所有 service 对象映射

    return {
        presto_sql: PrestoSqlServices,
        mongo_orm: MongoOrmServices,
    }
    """
    map_dic = dict()
    all_file_names = []
    dir_path = os.path.dirname(os.path.abspath(__file__))
    bond_yield_paths = glob.glob(f"{dir_path}/bond_yield/*_services.py")
    car_paths = glob.glob(f"{dir_path}/car/*_services.py")
    fund_paths = glob.glob(f"{dir_path}/fund/*_services.py")
    pig_paths = glob.glob(f"{dir_path}/pig/*_services.py")
    stock_paths = glob.glob(f"{dir_path}/stock/*_services.py")
    news_paths = glob.glob(f"{dir_path}/news/*_services.py")
    for item_paths in [bond_yield_paths, car_paths, fund_paths, pig_paths, stock_paths, news_paths]:
        for _ in item_paths:
            all_file_names.append(os.path.basename(_).replace(".py", ""))
    for module in all_file_names:
        if module.startswith("_"):
            continue
        try:
            # > 自动获取 所有文件中的 XxxxServices
            # module_obj = importlib.import_module(f"sharetop.core.bond_yield.{module}")
            package_name = module.replace("_services", "")
            module_obj = importlib.import_module(f"sharetop.core.{package_name}.{module}")
            services_cls_strs = [_ for _ in dir(module_obj) if _.endswith("Services")]

            for services_cls_str in services_cls_strs:
                # 驼峰->下划线 AaaBbServices -> aaa_bb
                services_cls_str_snake = _camel_to_snake(services_cls_str).replace("_services", "")
                map_dic[services_cls_str_snake] = getattr(module_obj, services_cls_str)
        except Exception:
            import traceback
            # traceback.print_exc()
            raise ValueError(f"从{module}文件中导入服务类异常: {traceback.format_exc()}")

    return map_dic


_MAP_DIC = _get_all_services_map()


class BasicTop:

    def __init__(self, token):
        self.token = token
        self.__class_map = _MAP_DIC

    def common_exec_func(self, exec_code: str, func_params=None, class_params=None):
        return_data = []
        if func_params is None:
            func_params = {}
        if class_params is None:
            class_params = {}
        try:
            class_code = exec_code.split("_to_")[0]
            if class_code not in self.__class_map.keys():
                self.__class_map = _get_all_services_map()
                raise ValueError(f"{class_code}不合法, 请输入 {list(self.__class_map.keys())} 中内容 !")
            cls_obj = self.__class_map[class_code]
            class_params.update({"token": self.token})
            func_obj = getattr(cls_obj(**class_params), f"{exec_code}")
            return_data = func_obj(**func_params)
        except Exception as err:
            import traceback
            traceback.print_exc()
            return_data = pd.DataFrame()
            return_data = dict(status="error", message=str(err), detail_message=str(traceback.format_exc()))
        finally:
            res_data = return_data
            # res_data = dict(status="success", data=return_data)
        return res_data

# if __name__ == '__main__':
#     token = "000"
#     sharetop_obj = BasicTop(token)
#     d = sharetop_obj.common_exec_func("bond_yield", "real_time", {}, {"bond_name": "gcny10"})
#     print(d)



import datetime
import copy
import pandas as pd
from ..utils import requests_obj
from .config import url, bond_yield_url, headers
from ..common.explain_change import exchange_explain_new
from ...crawl.settings import *


class BondYieldServices:
    def __init__(self, token, *args, **kwargs):
        self.token = token
        self.__url = url
        self.__headers = headers

    def bond_yield_to_real_time(self, **kwargs):
        is_explain = kwargs.get("is_explain", False)
        bond_name = kwargs.get("bond_code")
        bond_url = self.__url.format(bond_name=bond_name)
        r = requests_obj.get(bond_url, data={}, headers=self.__headers)
        data = r.text
        data = data.replace(f'var hq_str_globalbd_{bond_name}="', "").replace('";', "")
        data_list = data.split(",")
        bond_name = data_list[0]
        open_price = data_list[1]
        previous_close = data_list[2]
        now_price = data_list[3]
        high_price = data_list[4]
        low_price = data_list[5]
        percentage_change = float(data_list[7]) * 100
        price_change = data_list[8]
        update_time = datetime.datetime.fromtimestamp(int(data_list[11]))
        item = {"bond_name": bond_name, "open_price": open_price, "previous_close": previous_close,
                "now_price": now_price,
                "high_price": high_price, "low_price": low_price,
                "percentage_change": percentage_change, "price_change": price_change, "update_time": update_time}
        df = pd.DataFrame([item])
        return exchange_explain_new(df, is_explain)

    def bond_yield_to_kline_data(self, **kwargs):
        is_explain = kwargs.pop("is_explain", False)
        bond_yield_base_url_list = copy.deepcopy(base_url_list)
        bond_yield_base_url_list.append(bond_yield_url)
        headers = {"token": self.token}
        r = requests_obj.get("".join(bond_yield_base_url_list), data=kwargs, headers=headers)
        if isinstance(r, dict):
            return r
        data_json = r.json()
        df = pd.DataFrame(data_json['data'])
        return exchange_explain_new(df, is_explain)



import rich
import uuid
import pandas as pd
from retry import retry
from ..utils import to_numeric, requests_obj, validate_request
from .config import EastmoneyFundHeaders
from ...crawl.settings import *
from ..common.explain_change import exchange_explain


@validate_request
@retry(tries=3)
@to_numeric
def get_fund_base_info(token: str, fund_code: str, is_explain: bool = False) -> pd.Series:
    """
    获取基金的一些基本信息
    Parameters
    ----------
    fund_code : str
        6 位基金代码
    Returns
    -------
    Series
        基金的一些基本信息
        :param is_explain:
        :param fund_code:
        :param token:
    """
    str_uuid = str(uuid.uuid4()).upper()
    params = (
        ('FCODE', fund_code),
        ('deviceid', str_uuid),
        ('plat', 'Iphone'),
        ('product', 'EFund'),
        ('version', '6.3.8'),
    )
    json_response = requests_obj.get(''.join(fund_basic_url), params, headers=EastmoneyFundHeaders).json()
    columns = {
        'FCODE': '基金代码',
        'SHORTNAME': '基金简称',
        'ESTABDATE': '成立日期',
        'RZDF': '涨跌幅',
        'DWJZ': '最新净值',
        'JJGS': '基金公司',
        'FSRQ': '净值更新日期',
        'COMMENTS': '简介',
        'ENDNAV': '基金规模',
        'FEGMRQ': '基金规模更新时间',
        'RLEVEL_SZ': '最新评级'
    }
    items = json_response['Datas']
    items = {columns.get(k): v for k, v in items.items() if columns.get(k)}
    if not items:
        rich.print('基金代码', fund_code, '可能有误')
        return pd.Series(index=columns.values())

    s = pd.DataFrame([items])
    df = s.apply(lambda x: x.replace('\n', ' ').strip() if isinstance(x, str) else x)
    return exchange_explain(df, is_explain)



import copy
import pandas as pd
from .config import ship_indicators_url, ship_indicators_field_dict
from ...crawl.settings import *
from ..utils import to_numeric, requests_obj, parse_obj
from ..common.explain_change import exchange_explain


@to_numeric
def get_ship_data_common(token: str, ship_url: str, field_dict: dict, org_name: str, year: str, ship_indicators: str,
                         limit: int) -> pd.DataFrame:
    """
    :param ship_indicators:
    :param token:
    :param field_dict:
    :param ship_url:
    :param org_name:
    :param year: 字段对应关系
    :param limit:
    :return:
    """
    headers = {"token": token}
    copy_base_url_list = copy.deepcopy(base_url_list)
    copy_base_url_list.append(ship_url)
    data = {}
    if limit:
        data['limit'] = limit
    if org_name:
        data['org_name'] = org_name
    if year:
        data['year'] = year
    if ship_indicators:
        data['ship_indicators'] = ship_indicators
    r = requests_obj.get("".join(copy_base_url_list), data=data, headers=headers)
    if isinstance(r, dict):
        return r
    data_json = r.json()
    df = parse_obj.parse_god_cup_json(data_json, field_dict)
    return df


def get_ship_indicators(token: str, org_name: str = None, year: str = None, ship_indicators: str = None, limit: int = 100,
                is_explain: bool = False) -> pd.DataFrame:
    """
    获取猪周期的历史猪肉价格和猪粮比
    :param is_explain:
    :param ship_indicators:
    :param year:
    :param org_name:
    :param token:
    :param limit:
    :return:
    """
    return exchange_explain(get_ship_data_common(token, ship_indicators_url, ship_indicators_field_dict, org_name, year,
                                                 ship_indicators, limit),
                            is_explain)



import time
import rich
import json
import re
import copy
from retry.api import retry
from typing import Any, Callable, Dict, List, TypeVar, Union
from .cache import SEARCH_RESULT_DICT
from .config import Quote
from .config import SEARCH_RESULT_CACHE_PATH
import pandas as pd
from functools import wraps
from ..crawl import BaseRequest
from ..parser import BaseParse
from ..crawl.settings import base_url_list


F = TypeVar('F')
requests_obj = BaseRequest()
parse_obj = BaseParse()


def to_numeric(func: F) -> F:
    """
    将 DataFrame 或者 Series 尽可能地转为数字的装饰器
    Parameters
    ----------
    func : Callable
        返回结果为 DataFrame 或者 Series 的函数
    Returns
    -------
    Union[DataFrame, Series]
    """

    ignore = ['股票代码', '基金代码', '代码', '市场类型', '市场编号', '债券代码', '行情ID', '正股代码']

    @wraps(func)
    def run(*args, **kwargs):
        values = func(*args, **kwargs)
        if isinstance(values, pd.DataFrame):
            for column in values.columns:
                if column not in ignore:
                    values[column] = values[column].apply(convert)
        elif isinstance(values, pd.Series):
            for index in values.index:
                if index not in ignore:
                    values[index] = convert(values[index])
        return values

    def convert(o: Union[str, int, float]) -> Union[str, float, int]:
        if not re.findall('\d', str(o)):
            return o
        try:
            if str(o).isalnum():
                o = int(o)
            else:
                o = float(o)
        except:
            pass
        return o

    return run


def validate_request(func):
    def wrapper(*args, **kwargs):
        # 执行校验逻辑，可以是任何你需要的操作
        if len(args) == 0 and not kwargs.get("token"):
            return {"msg": "请填写有效token"}
        token = kwargs.get("token") if len(args) == 0 else args[0]
        headers = {"token": token}
        data = {}
        copy_base_url_list = copy.deepcopy(base_url_list)
        copy_base_url_list.append("/a_stock/base_data/detection")
        r = requests_obj.get("".join(copy_base_url_list), data=data, headers=headers)
        if isinstance(r, dict):
            return r
        data_json = r.json()
        msg = data_json['data'].get('msg', "")
        if msg == "success":
            # 执行被装饰的函数
            return func(*args, **kwargs)
        else:
            return {"msg": "校验失败，请确认token是否最新或者稍后再试或者联系管理员"}
    return wrapper




def search_quote_locally(keyword: str) -> Union[Quote, None]:
    """
    在本地里面使用搜索记录进行关键词搜索

    Parameters
    ----------
    keyword : str
        搜索词

    Returns
    -------
    Union[Quote,None]

    """
    q = SEARCH_RESULT_DICT.get(keyword)
    # NOTE 兼容旧版本 给缓存加上最后修改时间
    if q is None or not q.get('last_time'):
        return None
    last_time: float = q['last_time']
    # 缓存过期秒数
    max_ts = 3600 * 24 * 3
    now = time.time()
    # 缓存过期，在线搜索
    if (now - last_time) > max_ts:
        return None
    # NOTE 一定要拷贝 否则改变源对象
    _q = q.copy()
    # NOTE 一定要删除它 否则会构造错误
    del _q['last_time']
    quote = Quote(**_q)
    return quote


def search_quote(
        keyword: str, count: int = 1, use_local: bool = True
) -> Union[Quote, None, List[Quote]]:
    """
    根据关键词搜索以获取证券信息

    Parameters
    ----------
    keyword : str
        搜索词(股票代码、债券代码甚至证券名称都可以)
    count : int, optional
        最多搜索结果数, 默认为 `1`
    use_local : bool, optional
        是否使用本地缓存

    Returns
    -------
    Union[Quote, None, List[Quote]]

    """
    # NOTE 本地仅存储第一个搜索结果
    if use_local and count == 1:
        quote = search_quote_locally(keyword)
        if quote:
            return quote
    url = 'https://searchapi.eastmoney.com/api/suggest/get'
    params = (
        ('input', f'{keyword}'),
        ('type', '14'),
        ('token', 'D43BF722C8E33BDC906FB84D85E326E8'),
        ('count', f'{count}'),
    )
    json_response = requests_obj.get(url, params).json()
    items = json_response['QuotationCodeTable']['Data']
    if items is not None:
        quotes = [Quote(*item.values()) for item in items]
        # NOTE 暂时仅存储第一个搜索结果
        save_search_result(keyword, quotes[:1])
        if count == 1:
            return quotes[0]
        return quotes
    return None


def save_search_result(keyword: str, quotes: List[Quote]):
    """
    存储搜索结果到文件中

    Parameters
    ----------
    keyword : str
        搜索词
    quotes : List[Quote]
        搜索结果
    """
    with open(SEARCH_RESULT_CACHE_PATH, 'w', encoding='utf-8') as f:
        # TODO考虑如何存储多个搜索结果
        for quote in quotes:
            now = time.time()
            d = dict(quote._asdict())
            d['last_time'] = now
            SEARCH_RESULT_DICT[keyword] = d
            break
        json.dump(SEARCH_RESULT_DICT.copy(), f)


@retry(tries=3, delay=1)
def get_quote_id(stock_code: str) -> str:
    """
    生成东方财富股票专用的行情ID

    Parameters
    ----------
    stock_code : str
        证券代码或者证券名称

    Returns
    -------
    str
        东方财富股票专用的 secid
    """
    if len(str(stock_code).strip()) == 0:
        raise Exception('证券代码应为长度不应为 0')
    quote = search_quote(stock_code)
    if isinstance(quote, Quote):
        return quote.quote_id
    if quote is None:
        rich.print(f'证券代码 "{stock_code}" 可能有误')
        return stock_code


def process_dataframe_and_series(
    function_fields: Dict[str, Callable] = dict(),
    remove_columns_and_indexes: List[str] = list(),
):
    """
    对 DataFrame 和 Series 进一步操作

    Parameters
    ----------
    function_fields : Dict[str, Callable], optional
        函数字典
    remove_columns_and_indexes : List[str], optional
        需要删除的行或者列, by default list()
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            values = func(*args, **kwargs)
            if isinstance(values, pd.DataFrame):
                for column, function_name in function_fields.items():
                    if column not in values.columns:
                        continue
                    values[column] = values[column].apply(function_name)
                for column in remove_columns_and_indexes:
                    if column in values.columns:
                        del values[column]
            elif isinstance(values, pd.Series):
                for index in remove_columns_and_indexes:
                    values = values.drop(index)
            return values

        return wrapper

    return decorator





import pandas as pd
from ..utils import to_numeric, requests_obj, validate_request
from retry import retry
from .config import EastmoneyFundHeaders
from ...crawl.settings import *
from ..common.explain_change import exchange_explain


@validate_request
@retry(tries=3)
@to_numeric
def get_fund_history_price(token: str, fund_code: str, pz: int = 40000, is_explain: bool = False) -> pd.DataFrame:
    """
    根据基金代码和要获取的页码抓取基金净值信息

    Parameters
    ----------
    fund_code : str
        6 位基金代码
    pz : int, optional
        页码, 默认为 40000 以获取全部历史数据

    Returns
    -------
    DataFrame
        包含基金历史净值等数据

    Examples
    --------
        日期    单位净值    累计净值     涨跌幅
    0    2021-06-11  1.5188  3.1499   -3.09
    1    2021-06-10  1.5673  3.1984    1.69
    2    2021-06-09  1.5412  3.1723    0.11
    3    2021-06-08  1.5395  3.1706    -6.5
    4    2021-06-07  1.6466  3.2777    1.61
    ...         ...     ...     ...     ...
    1469 2015-06-08  1.0380  1.0380  2.5692
    1470 2015-06-05  1.0120  1.0120  1.5045
    1471 2015-06-04  0.9970  0.9970      --
    1472 2015-05-29  0.9950  0.9950      --
    1473 2015-05-27  1.0000  1.0000      --
    :param is_explain:
    :param pz:
    :param fund_code:
    :param token:

    """
    data = {
        'FCODE': f'{fund_code}',
        'IsShareNet': 'true',
        'MobileKey': '1',
        'appType': 'ttjj',
        'appVersion': '6.2.8',
        'cToken': '1',
        'deviceid': '1',
        'pageIndex': '1',
        'pageSize': f'{pz}',
        'plat': 'Iphone',
        'product': 'EFund',
        'serverVersion': '6.2.8',
        'uToken': '1',
        'userId': '1',
        'version': '6.2.8',
    }
    json_response = requests_obj.get(''.join(fund_history_url), data, headers=EastmoneyFundHeaders).json()
    rows = []
    columns = ['日期', '单位净值', '累计净值', '涨跌幅']
    if json_response is None:
        return pd.DataFrame(rows, columns=columns)
    datas = json_response['Datas']
    if len(datas) == 0:
        return pd.DataFrame(rows, columns=columns)
    for stock in datas:
        date = stock['FSRQ']
        rows.append(
            {
                '日期': date,
                '单位净值': stock['DWJZ'],
                '累计净值': stock['LJJZ'],
                '涨跌幅': stock['JZZZL'],
            }
        )
    df = pd.DataFrame(rows)
    return exchange_explain(df, is_explain)



import json
from pathlib import Path
from typing import Dict

import pandas as pd
from .config import SEARCH_RESULT_CACHE_PATH

# 关键词搜索缓存
SEARCH_RESULT_DICT: Dict[str, dict] = dict()
# 行情ID搜索缓存
BASE_INFO_CACHE: Dict[str, pd.Series] = dict()
path = Path(SEARCH_RESULT_CACHE_PATH)
if path.exists():
    load_success = False
    with path.open('r', encoding='utf-8') as f:
        try:
            SEARCH_RESULT_DICT = json.load(f)
            load_success = True
        except:
            pass
    if not load_success:
        path.open('w').close()



from typing import Dict, Optional

from ftplib import FTP

# 创建FTP对象
# ftp = FTP()
#
# # 连接FTP服务器
# ftp.connect('183.6.56.70', 2121)
#
# # 登录FTP服务器
# ftp.login('ftp-ths-read', 'i!9|j3pB')
#
# ftp.cwd("/2023-12-25/insert")
#
# # 执行FTP操作
# # 例如，列出FTP服务器上的文件
# ftp.retrlines('LIST')
# # 关闭连接
# ftp.quit()



# def get_filelist(hostname, username, password):
#     # 连接到FTP服务器
#     try:
#         ftp = FTP()
#         ftp.connect(hostname, 2121)
#         ftp.login(username, password)
#         # 切换到指定目录
#         directory = '/2023-12-25/insert'  # 将此路径更改为所需的目录路径
#         ftp.cwd(directory)
#         # 获取当前目录下的文件列表
#         file_list = []
#         files = ftp.nlst()
#         for filename in files:
#             if not filename.startswith('.'):  # 如果不是隐藏文件则添加到列表中
#                 print("filename=======:", filename)
#                 # localfile = open(filename, "wb")
#                 # ftp.retrbinary(f"RETR {filename}", localfile.write)
#                 file_list.append(filename)
#         print("file_list=:", file_list)
#         for _ in file_list:
#             localfile = open(_, "wb")
#             ftp.retrbinary(f"RETR {_}", localfile.write)
#         return file_list, ftp
#     except Exception as e:
#         print("Error occurred while connecting to the FTP server.")
#         print(e)
#         return None
#     finally:
#         ftp.quit()

# 设置FTP服务器信息
# hostname = '183.6.56.70'
# username = 'ftp-ths-read'
# password = 'i!9|j3pB'

# 调用函数并打印结果
# result, ftp = get_filelist(hostname, username, password)
# if result is not None:
#     for filename in result:
#         print(filename)
#         # filename = "example.txt"
#         # path = f"/2023-12-25/insert/{filename}"
#         # print('path:', path)
#         # localfile = open(filename, "wb")
#         # ftp.retrbinary(f"RETR {filename}", localfile.write)
# else:
#     print("Failed to retrieve file list from FTP server.")
class FtpServe:
    def __init__(self, hostname: str, username: str, password: str, port: Optional[int] = 2121):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.port = port
        self.ftp = self.ftp_obj()

    def ftp_obj(self):
        ftp = FTP()
        ftp.connect(self.hostname, self.port)
        ftp.login(self.username, self.password)
        return ftp

    def get_file(self, directory: str):
        # directory = '/2023-12-25/insert'  # 将此路径更改为所需的目录路径
        self.ftp.cwd(directory)
        # 获取当前目录下的文件列表
        file_list = []
        files = self.ftp.nlst()
        for filename in files:
            if not filename.startswith('.'):  # 如果不是隐藏文件则添加到列表中
                print("filename=======:", filename)
                # localfile = open(filename, "wb")
                # ftp.retrbinary(f"RETR {filename}", localfile.write)
                file_list.append(filename)
        print("file_list=:", file_list)
        for _ in file_list:
            localfile = open(_, "wb")
            # self.ftp.retrbinary(f"RETR {_}", localfile.write)
            self.ftp.retrlines(f"LIST {_}")


    def run(self, time_path: str):
        directory = f"{time_path}/insert"
        self.get_file(directory)



if __name__ == '__main__':
    hostname = '183.6.56.70'
    username = 'ftp-ths-read'
    password = 'i!9|j3pB'
    run_obj = FtpServe(hostname, username, password)
    time_path = "2023-12-26"
    run_obj.run(time_path)



from pathlib import Path

HERE = Path(__file__).parent

a_stock_list_url = "/stock/a_stock/stock_list"

a_stock_trade_date_url = "/stock/a_stock/trade_cal_list"

a_stock_index_url = "/stock/a_stock/stock_index"

stock_intl_index_url = "/stock/index/stock_intl_index"

index_stock_composition_url = "/stock/index/stock_composition"

stock_bonus_url = "/stock/stock_bonus"

stock_repurchase_url = "/stock/stock_repurchase"





# 股票基本信息表头
EASTMONEY_STOCK_BASE_INFO_FIELDS = {
    'f57': '股票代码',
    'f58': '股票名称',
    'f162': '市盈率(动)',
    'f167': '市净率',
    'f127': '所处行业',
    'f116': '总市值',
    'f117': '流通市值',
    'f198': '板块编号',
    'f173': 'ROE',
    'f187': '净利率',
    'f105': '净利润',
    'f186': '毛利率',
}

EASTMONEY_STOCK_DAILY_BILL_BOARD_FIELDS = {
    'SECURITY_CODE': '股票代码',
    'SECURITY_NAME_ABBR': '股票名称',
    'TRADE_DATE': '上榜日期',
    'EXPLAIN': '解读',
    'CLOSE_PRICE': '收盘价',
    'CHANGE_RATE': '涨跌幅',
    'TURNOVERRATE': '换手率',
    'BILLBOARD_NET_AMT': '龙虎榜净买额',
    'BILLBOARD_BUY_AMT': '龙虎榜买入额',
    'BILLBOARD_SELL_AMT': '龙虎榜卖出额',
    'BILLBOARD_DEAL_AMT': '龙虎榜成交额',
    'ACCUM_AMOUNT': '市场总成交额',
    'DEAL_NET_RATIO': '净买额占总成交比',
    'DEAL_AMOUNT_RATIO': '成交额占总成交比',
    'FREE_MARKET_CAP': '流通市值',
    'EXPLANATION': '上榜原因',
}
